***********************************************************************
* Program....: DSETBASE.PRG
* Author.....: Andy Kramek
* Date.......: November 17, 2000
* Purpose....: Root Class Definition for a DataSet
***********************************************************************

***********************************************************************
*** Root Data Set Object Definition
***********************************************************************
DEFINE CLASS xDataSet AS xObjBase

  *******************************************************************
  *** Custom Property Definitions
  *******************************************************************
	*** Array to hold the names of all current cursors
	DIMENSION aCursors[1] 
	*** Collection which holds the items associated with this entity.
	DIMENSION aItems[1] 
	*** Number of Cursors currently held by this object
	nCursorCount = 0
	*** Number of items in the Entity Collection
	nitemcount = 0
	*** Object Reference to Data Manager
	oDM = NULL
	*** Protect the properties which are essential to the internal workings
  PROTECTED aCursors, nCursorCount, aItems, nItemCount, oDM

  *******************************************************************
  *** [P] ADDERRORS(): Change in behavior from normal for this class
  *** because it will receive an error object from the Data Manager
  *** So, here, we just want to transfer error information.
  *******************************************************************
  PROTECTED FUNCTION AddErrors( toObject )
    LOCAL loErrObj, lnErrs
    *** Check Parameters
    IF VARTYPE( toObject ) # 'O'
      ASSERT .F. MESSAGE 'This is another Brain Dead Programmer Error!' + CHR( 13 ) +;
           'You MUST pass an object reference to the AddErrors Method!' + CHR( 13 ) +;
           'Have a nice day now...'
      RETURN .F.
    ENDIF
    WITH This
      *** Has it got any errors?
      lnErrs = toObject.nErrorCount
      IF lnErrs > 0
        *** If so, add them to the current Collection
        FOR lnCnt = 1 TO lnErrs
          .LogError( toObject.aErrors[lnCnt, 1], ;
                     toObject.aErrors[lnCnt, 2], ;
                     toObject.aErrors[lnCnt, 3] )
        ENDFOR
      ENDIF
    ENDWITH
    RETURN
  ENDFUNC

  *******************************************************************
  *** [P] ADDCURSOR(): Protected method to Add a cursor object of the 
  *** required class and name and add the cursor name to the 
  *** object's cursor collection
  *******************************************************************
  PROTECTED FUNCTION AddCursor( tcCursorClass, tcCurName  )
    LOCAL llRetVal, lcName, lcCur, loCur
    WITH This
      *** Check that we got something potentially valid
      llRetVal = .ValidateParam( tcCurName, "C" ) AND .ValidateParam( tcCursorClass, "C" )
      IF ! llRetVal
        RETURN llRetVal
      ENDIF
      *** Get the name into a variable first
      lcName = LOWER( ALLTRIM( tcCurName ))
      *** Add the appropriate cursor object, and a reference to it
      .AddProperty( lcName, NULL )
      lcCur = "This." + lcName
      &lcCur = CREATEOBJECT( tcCursorClass )
      *** Check that we have an Object
      IF TYPE( lcCur ) # "O"
        *** We have an error - Get log it!
        ASSERT .F. MESSAGE "Unable to Add Object for Cursor " + tcCursor
        .LogError( 9003, tcCursor, LOWER( PROGRAM() ) )
        llRetVal = .F.    && Return Error Code
      ELSE
        *** We got an object so default everything to name for now
        loCur = EVAL( lcCur )
        WITH loCur
          .Alias   = lcName
          .Name    = lcName
        ENDWITH
        *** Unless this is a lookup cursor
        IF ! 'LKUP' $ UPPER( lcName )
          *** Add the cursor name to the collection
          .nCursorCount = .nCursorCount + 1
          DIMENSION .aCursors[ .nCursorCount ]
          .aCursors[ .nCursorCount ] = lcName
        ENDIF
        llRetVal = .T.
      ENDIF
    ENDWITH
    *** Return Status
    RETURN llRetVal
	ENDFUNC

  *******************************************************************
  *** [P] AFTERFAILSAVE(): Template Method called by TryUpdate after save fails
  *******************************************************************
  PROTECTED FUNCTION AfterFailSave

  ENDFUNC

  *******************************************************************
  *** [P] AFTERGOODSAVE(): Template Method called by TryUpdate after save succeeds
  *******************************************************************
  PROTECTED FUNCTION AfterGoodSave

  ENDFUNC

  *******************************************************************
  *** [P] BEFORESAVE(): Template Method called by TryUpdate before any action
  *** is initiated. Save will be aborted if this method returns .F.
  *******************************************************************
  PROTECTED FUNCTION BeforeSave( tcCursorList )
        
  ENDFUNC
    
  ********************************************************************
  *** [E] DO(): Preprocessing metod for validating and accessing functionality
  ********************************************************************
  FUNCTION Do( tcAction, tcCursor, tuParm1, tuParm2, tuParm3)
  LOCAL lnSelect, loCursor, lcWhere, luRetVal
    *** Save Work Area
    lnSelect = SELECT()
    *** Initialize RETURN Value
    luRetVal = ""
    *** Clean Parameter
    tcAction = UPPER( ALLTRIM( tcAction ))
    DO CASE
      CASE tcAction = "QUERY"
        *** tuParm1 = Ad-Hoc WHERE clause
        IF TYPE( "This." + tcCursor ) # "O"
          This.LogError( 9005, tcCursor, LOWER( PROGRAM() ) )
          luRetVal = NULL
        ELSE
          *** Pass - on object ref to cursor and action to the data manager
          loCursor = EVAL( 'This.' + tcCursor )
          *** Did we get a Filter condition too
          lcWhere = IIF( This.ValidateParam( tuParm1, "C"), ALLTRIM( tuParm1 ), "" )
          *** Pass on the Filter
          luRetVal = This.ProcessRequest( 'Query', loCursor, lcWhere)
        ENDIF

      CASE tcAction = "REQUERY" 
        *** tuParm1 = Ad-Hoc WHERE clause
        IF TYPE( "This." + tcCursor ) # "O"
          This.LogError( 9005, tcCursor, LOWER( PROGRAM() ) )
          luRetVal = NULL
        ELSE
          *** Pass - on object ref to cursor and action to the data manager
          loCursor = EVAL( 'This.' + tcCursor )
          *** Pass on the Filter
          luRetVal = This.ProcessRequest( 'ReQuery', loCursor )
        ENDIF

      CASE tcAction = "GETLASTID"
        IF TYPE( "This." + tcCursor ) # "O"
          This.LogError( 9005, tcCursor, LOWER( PROGRAM() ) )
          luRetVal = NULL
        ELSE
          *** Pass - on object ref to cursor and action to the data manager
          loCursor = EVAL( 'This.' + tcCursor )
          *** Pass on the Filter
          luRetVal = This.ProcessRequest( 'GetLastID', loCursor )
        ENDIF

      CASE tcAction = "LOOKUP"
        *** Build the Where clause here
        *** tuParm1 = Value to find
        *** tuParm2 = Field to Search
        *** tuParm3 = Field to return
        lcWhere = ALLTRIM( tuParm2 ) + " LIKE " + This.ValToStr( tcCursor, tuParm1 )
        *** Now execute the query
        luRetVal = This.LookUpExp( tcCursor, lcWhere, tuParm3)

      CASE tcAction = "FETCHVAL"
        *** tuParm1 = Ad-Hoc Where clause
        *** tuParm2 = Field to Return
        *** Pass it on
        luRetVal = This.LookUpExp( tcCursor, tuParm1, tuParm2 )
            
      OTHERWISE
        *** We cannot handle that action just now
        This.LogError( 9008, tcAction, LOWER( PROGRAM() ) )
        luRetVal = NULL
    ENDCASE
    *** Check for errors
    IF ISNULL( luRetVal )
      This.ShowErrors()
    ENDIF
    *** Tidy up here
    SELECT (lnSelect)
    RETURN luRetVal
  ENDFUNC

  *******************************************************************
  *** [P] LOOKUPEXP(): Returns the value from a specified field based 
  ***          : on a specified WHERE clause
  *******************************************************************
  PROTECTED FUNCTION LookupExp( tcCursor, tcWhere, tcRetField )
    LOCAL luRetVal, loCursor
    IF PCOUNT() # 3
    	 *** Must pass ALL parameters
      ASSERT .F. MESSAGE "Must pass ALL parameters to DoLookUp()"
      RETURN ""
    ENDIF
    *** If the Cursor is NOT an object, add an object
    IF TYPE( "This." + tcCursor ) # "O"
    	*** Just Add One of the same name
    	luRetVal = This.AddCursor( 'xCurSQL', tcCursor )
    	*** If it failed, return
    	IF ! luRetVal
    		RETURN luRetVal
    	ELSE
    		*** Set the cursor & table name the same as the cursor name
    		loCursor = EVAL( 'This.' + tcCursor )
    		WITH loCursor
    			.cTables = tcCursor
    			.Alias = tcCursor
    			*** And make it read only
    			.lUpdatable = .F.
    		ENDWITH
    	ENDIF
    ELSE
    	*** Pass - on object ref to cursor and action to the data manager
    	loCursor = EVAL( 'This.' + tcCursor )
    ENDIF
    *** Now execute the query
    luRetVal = This.ProcessRequest( 'LookUpExp', loCursor, tcWhere, tcRetField )
    *** And return the result
    RETURN NVL( luRetVal, "" )
  ENDFUNC

  *******************************************************************
  *** [P] SETUPREFTABLES(): Open up the standard reference tables for the object
  *******************************************************************
  PROTECTED FUNCTION SetUpRefTables
    This.OpenLocalTable("curdefs")
    This.OpenLocalTable("datasets")
    This.OpenLocalTable("dsetlink")
    This.OpenLocalTable("curnames")
    This.OpenLocalTable("connects" )
  ENDFUNC

  ********************************************************************
  *** [E] SHOWERRORS(): Retrieve and display Errors
  *** USE ONLY WHEN DEBUGGING - NEVER CALLED AT RUN TIME
  ********************************************************************
  FUNCTION ShowErrors()
  LOCAL lnSelect, loErr, lcTxt, lcDispTxt
    *** Save Work Area
    lnSelect = SELECT()
    loErr = This.GetErrors()
    lcTxt = ""
    IF loErr.nErrorCount = 0
      *** No errors to display
      lcDispTxt = "No Errors were recorded"
      MESSAGEBOX( lcDispTxt, 64, "Error Report")
      RETURN
    ELSE
      FOR lnCnt = 1 TO loErr.nErrorCount
        lcTxt = lcTxt + IIF( NOT EMPTY( lcTxt ), CHR(13)+CHR(10), "" )
        *** Error Number
        lcTxt = lcTxt + ALLTRIM( TRANSFORM( loErr.aErrors[ lnCnt,1 ] )) + ": "
        lcTxt = lcTxt + ALLTRIM( loErr.aErrors[ lnCnt, 2 ] )
        IF ! EMPTY( loErr.aErrors[ lnCnt, 3 ] )
          lcTxt = lcTxt + " (" + ALLTRIM( loErr.aErrors[ lnCnt, 3 ] ) + ")"
        ENDIF
      NEXT
      lcDispTxt = lcTxt + CHR(13) + CHR(10) + CHR(13) + CHR(10)
      lcDispTxt = lcDispTxt + "Save these errors to FILE (ErrTxt.txt)?"
    ENDIF    
    *** Display Error Messages
    IF MESSAGEBOX( lcDispTxt, 32 + 4 + 256, "Error Report") = 6 &&YES
      *** Save to file if required
      lcTxt = "Errors Recorded at: " + TTOC( DATETIME() ) + CHR(13) + CHR(10) + lcTxt 
      STRTOFILE( lcTxt + CHR(13) + CHR(10), "errTxt.txt", .T. )
    ENDIF
    *** Tidy Up
    SELECT (lnSelect)
    RETURN
  ENDFUNC

  *******************************************************************
  *** [E] TRYUPDATE(): Exposed Method to save one or more cursors
  *******************************************************************
  FUNCTION TryUpdate( tcCursorList )
  *** Expects to receive a character string containing one or more cursor names
  *** If multiple cursor names are passed, they must be separated by commas.
  LOCAL lcCursor, lnDS, lnCnt, llOk, llTUOk
  *** Clean parameters
  tcCursorList = IIF( This.ValidateParam( tcCursorList, 'C'), tcCursorList, '' )
  IF EMPTY( tcCursorList )
    *** Nothing Passed! Assume ALL updatable cursors
    tcCursorList = This.ListCursors( .T. )
  ENDIF
  *** Call BeforeSave
  llOk = This.BeforeSave( tcCursorList )
  IF ! llOk
    This.LogError( 9013, 'BeforeSave()', LOWER( PROGRAM() ) )
    RETURN llOk
  ENDIF
  *** Counter for cursor
  STORE 0 TO lnCnt
  STORE .T. TO llOk, llTUOK
  *** Start a Front End Transaction
  BEGIN TRANSACTION
  *** Loop - thru and update all cursors
  DO WHILE llOk
    lnCnt = lnCnt + 1
    *** Extract cursor from list
    lcCursor = This.GetItem( tcCursorList, lnCnt )
    *** Exit when no more cursors found
    IF ISNULL( lcCursor )
      EXIT
    ENDIF
    *** If the current cursor is empty ignore.
    IF RECCOUNT( lcCursor ) == 0
      llOk = llOk AND .T.
      LOOP
    ENDIF
    *** Get an object Reference to the cursor
    IF TYPE( "EVAL( 'This.' + lcCursor )" ) = "O"
      loCursor = EVAL( "This." + lcCursor )
  	  llTUOk = This.ProcessRequest( 'TableUpdate', loCursor  )
  	  *** Report failure in Dev Mode
  	  IF ! llTUOk
  	  	ASSERT .F. MESSAGE "Cannot Update data from cursor '" + lcCursor + "'"
  	  	This.LogError( 9020, lcCursor, LOWER( PROGRAM() ))
  	  ELSE
  	    *** Tablupdate the front end cursor so as to lose changes
  	    *** We can force this one because the back end save has succeeded 
  	    llTUOK = TABLEUPDATE( 2, .T., lcCursor )
  	  ENDIF
  	  llOk = llOk AND llTuOK
	  ELSE
  		This.LogError( 9005, lcCursor, LOWER( PROGRAM() ) )	
    	ENDIF
    ENDDO
    *** Commit/Rollback on the back end first
    lcClose = IIF( llOK, "Commit", "RollBack" )
    llTUOk = This.ProcessRequest( lcClose )
    *** Kill the Front End Transaction 
    IF llTUOk
      *** Good Save!
      END TRANSACTION
      This.AfterGoodSave()
    ELSE
      *** Failed!
      ROLLBACK
      This.AfterFailSave()
    ENDIF
    *** Return success/fail to caller
    RETURN llOk
  ENDFUNC

  *******************************************************************
  *** [E] VALTOSTR(): Exposed Method to convert a specified value into 
  *** the SQL Server compatible string equivalent
  *******************************************************************
  FUNCTION ValToStr( tcCursor, tuExp, tcType )
    LOCAL loCursor, lcMethodName, lcRetVal
    *** Do we have a cursor name
    IF TYPE( "This." + tcCursor ) = "O"
      loCursor = EVAL( 'This.' + tcCursor )
      lcMethodName = "ValTo" + SUBSTR( loCursor.cConnName, 4, 3 ) + "Str"
    ELSE
      *** Use SQL style by default
      lcMethodName = "ValToSqlStr"
    ENDIF
    IF ! PEMSTATUS( This.oDM, lcMethodName, 5 )
      This.LogError( 9007, lcMethodName, LOWER( PROGRAM() ))
      RETURN NULL
    ENDIF
    lcRetVal = EVAL( "This.oDM." + lcMethodName + "( tuExp, tcType )" )
    RETURN lcRetVal
  ENDFUNC

  *******************************************************************
  *** [P] GETOBJDETAILS(): Retrieves the names and locations of the 
  *** components of an object
  *******************************************************************
  PROTECTED FUNCTION GetObjDetails
    LOCAL lcName, lnEntPK, lnCnt, llRetVal
    *** Have we got this entity name
    lcName = UPPER( ALLTRIM( This.Name ))
    IF ! SEEK( lcName, "datasets", "dsetname" )
      This.LogError( 9010, lcName, LOWER( PROGRAM() ) )
      RETURN .F.
    ENDIF
    *** YES! Get the PK for the entity
    lnEntPK = datasets.dsetpk
    *** And populate the Collection
    SELECT CN.curlocn, CN.curname, CN.curType, CN.DefFK ;
      FROM datasets DS, dsetlink DL, curnames CN ;
     WHERE DS.dsetpk = DL.dsetfk ;
       AND DL.curfk = CN.curpk ;
       AND DS.dsetpk = lnEntPK ;
      INTO ARRAY This.aItems
    *** And the Item counter
    This.nItemCount = _TALLY
    FOR lnCnt = 1 TO This.nItemCount
      *** Only do Cursors!
      IF INLIST( ALLTRIM( This.aItems[3] ), "WKSH", "CALC" )
        LOOP
      ENDIF
      *** It is a cursor so open it up - pass name of the file containing the definition
      llRetVal = This.SetUpCursor( UPPER( ALLTRIM( This.aItems[lnCnt,1] )), ;
                                   UPPER( ALLTRIM( This.aItems[lnCnt,2] )), ;
                                   This.aItems[lnCnt,4])
      *** If it failed to open - bomb out!
    	IF ! llRetVal
    		This.Logerror( 9004, ALLTRIM( This.aItems[2] ), LOWER( PROGRAM() ))
    		RETURN llRetVal
    	ENDIF
    NEXT
    *** Return Status
    RETURN llRetVal
  ENDFUNC
  
  *******************************************************************
  *** [E] INIT(): Base Class Method - overridden here
  *******************************************************************
  FUNCTION Init( tuSetUpParams )
    DODEFAULT()
    *** Default Object Name to Class Name
    This.Name = This.Class
    *** Pass through the parameters received
    IF ! This.InitObj( tuSetUpParams )
      ******************************************
      *** For Debugging enable this block of code
      ******************************************
      *** IF VERSION(2) # 0
      ***   *** Only do this if we are in Dev Mode
      ***   This.ShowErrors()
      *** ENDIF
      ******************************************
    ENDIF
  ENDFUNC

  *******************************************************************
  *** [P] INITOBJ(): Called from Init() to open the local VFP Tables 
  *** that will be needed for any subsequent operation. If successful 
  *** calls the Instance Level SetUpObject() method
  *******************************************************************
  PROTECTED FUNCTION InitObj( tuSetUpParams )
    LOCAL llRetVal
    *** Set Data Manager Reference if we have it
    IF VARTYPE( _Screen.oDM ) = "O"
      This.oDM = _Screen.oDM
    ENDIF
    WITH This
      *** Open Reference Tables if not already in use
      .SetUpRefTables()
      *** If we've not got a reference to the Data Manager, create one
      IF VARTYPE( This.oDM ) # "O"
      	  *** Try and Create the Data Manager using a default connection
         	.oDM = CREATEOBJECT( 'xDatMgr', "DEFAULT" )
        *** Check that DM was created
        IF TYPE( "This.oDM" ) # "O" OR ISNULL( This.oDM )
          This.LogError( 9001, , LOWER( PROGRAM() ))
          RETURN .F.
        ENDIF
      ENDIF
      *** Populate the aItems Collection with details of the items for this entity
      llRetVal = This.GetObjDetails()
      IF NOT lLRetVal
        This.LogError( 9010, , LOWER( PROGRAM() ))
      ELSE
        *** If OK Call the Instance Level Set-Up Method
        llRetVal = .SetUpObject( tuSetUpParams )
      ENDIF
      RETURN llRetVal
    ENDWITH
  ENDFUNC
  
  *******************************************************************
  *** [P] LISTCURSORS(): Build a comma separated list of valid cursors 
  ***	from the aCursors collection
  *******************************************************************
  	PROTECTED FUNCTION ListCursors( tlUpDatableOnly )
    LOCAL lcRetVal, loCursor, lcCur
    *** Initialise the return string
    lcRetVal = ""
    IF This.nCursorCount > 0
      *** We have some cursors registered
      FOR EACH lcCur IN This.aCursors
        *** Check that cursor is still valid
        loCursor = EVAL( "This." + lcCur )
        IF USED( loCursor.Alias )
          IF tlUpdatableOnly
            IF loCursor.lUpdatable
              *** Add it to the List of Cursors
              lcRetVal = lcRetVal + IIF( !EMPTY(lcRetVal), ",","") + lcCur
            ENDIF
          ELSE
            *** Add it to the List of Cursors anyway
            lcRetVal = lcRetVal + IIF( !EMPTY(lcRetVal), ",","") + lcCur
          ENDIF
        ELSE
          *** The cursor is closed for some reason!!!!
          ASSERT .F. MESSAGE "Cursor " + lcCur + " is registered but not available"
        ENDIF
      NEXT
    ENDIF
    *** Return list of cursors
    RETURN lcRetVal
	ENDFUNC

  *******************************************************************
  *** [P] PROCESSREQUEST(): Passes on the various actions to the Data Manager 
  *******************************************************************
  PROTECTED FUNCTION ProcessRequest( tcAction, toCursor, tuParm1, tuParm2, tuParm3, tuParm4, tuParm5 )
    LOCAL loRetVal, luRetVal
    tcAction = UPPER( ALLTRIM( tcAction ) )
    DO CASE
      CASE PCOUNT() = 1
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction )
      CASE PCOUNT() = 2
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor )
      CASE PCOUNT() = 3
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor, tuParm1 )
      CASE PCOUNT() = 4
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor, tuParm1, tuParm2 )
      CASE PCOUNT() = 5
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor, tuParm1, tuParm2, tuParm3 )
      CASE PCOUNT() = 6
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor, tuParm1, tuParm2, tuParm3, tuParm4 )
      OTHERWISE
        loRetVal = This.oDM.Do( SET( 'DATASESSION' ), tcAction, toCursor, tuParm1, tuParm2, tuParm3, tuParm4, tuParm5 )
    ENDCASE
    
    *** Return value to calling program
    IF VARTYPE( loRetVal ) = "O"
      IF loRetVal.nErrorCount > 0
        *** An error occurred in the Data Manager
        *** So add it to the DataSet error collection
        This.AddErrors( loRetVal )             
      ENDIF
      *** And return whatever the DM passed us back
      luRetVal = loRetVal.uRetVal
    ELSE
      *** Not sure what we got, but just pass it through!
      luRetVal = loRetVal
    ENDIF
    *** Now just return whatever we have
    RETURN luRetVal
  ENDFUNC
	
  *******************************************************************
  	*** [P] SETUPCURSOR(): Reads values for a standard Data Class cursor 
	  *** from the current record in cursor definition table
  *******************************************************************
  PROTECTED FUNCTION SetupCursor( tcSceFile, tcCursor, tnDefFK )
    LOCAL lcSceFile, lcAlias, llRetVal, loCur, lcCurName
    *** This is used for creating UI Cursors, so we need to access the appropriate 
    *** cursor definition Table. Default to 'curdefs.dbf' if nothing is specified
    lcSceFile = IIF( This.ValidateParam( tcSceFile, "C"), tcSceFile, 'curdefs.dbf' )
    llRetVal = This.OpenLocalTable( lcSceFile )
    IF ! llRetVal
      This.LogError( 9009, lcSceFile, LOWER( PROGRAM() ) )
      RETURN llRetVal
    ENDIF
    *** If we get this far, we have the correct file open, so select it
    lcAlias = JUSTSTEM( lcSceFile )
    SELECT (lcAlias )
    *** Now Create the Cursor according to the definition
    *** We MUST have an index named "curpk" on whatever definition table is used 
    IF SEEK( tnDefFK, lcAlias, 'curpk' )
      lcCurName = ALLTRIM( tcCursor )
      *** Add the cursor object
      This.AddCursor( ALLTRIM( cCurClass ), lcCurName )
      *** Populate properties
      loCur = EVALUATE( "This."  + lcCurName )
      *** Strip out any CHR(10), CHR(13) or CHR(0) characters
      WITH loCur
        *** Set the Connection and Behavior Properties. cCurClass MUST always be in the form
        *** xCurXXXXX where "XXXXX" is the suffix of the Behavior and Connection class names
        .cBehaviour  = 'BHV' + ALLTRIM( SUBSTR( cCurClass, 5 ))
        .cConnName   = 'CON' + ALLTRIM( SUBSTR( cCurClass, 5 ))
         *** Read all other settings from the CurDef Table
        .cSelFields  = CHRTRAN( ALLTRIM( cSelFields ), CHR(0)+CHR(13)+CHR(10),'')
        .cTables     = CHRTRAN( ALLTRIM( cTables )   , CHR(0)+CHR(13)+CHR(10),'')
        .database 	 = CHRTRAN( ALLTRIM( cDatabase ) , CHR(0)+CHR(13)+CHR(10),'')
        .cPermWhere  = CHRTRAN( ALLTRIM( cPermWhere ), CHR(0)+CHR(13)+CHR(10),'')
        .cPermJoins  = CHRTRAN( ALLTRIM( cPermJoins ), CHR(0)+CHR(13)+CHR(10),'')
        .cPermHaving = CHRTRAN( ALLTRIM( cPermHave ) , CHR(0)+CHR(13)+CHR(10),'')
        .cDefOrder   = CHRTRAN( ALLTRIM( cDefOrder ) , CHR(0)+CHR(13)+CHR(10),'')
        .cDefGroupBy = CHRTRAN( ALLTRIM( cDefGRoup  ), CHR(0)+CHR(13)+CHR(10),'')
        .cUpdTable   = CHRTRAN( ALLTRIM( cUpdTable ) , CHR(0)+CHR(13)+CHR(10),'')
        .cKeyField   = CHRTRAN( ALLTRIM( cKeyField ) , CHR(0)+CHR(13)+CHR(10),'')
        .cUpdFields  = CHRTRAN( ALLTRIM( cUpdFields ), CHR(0)+CHR(13)+CHR(10),'')
        .lUpdatable  = lUpdatable
        .lPopulate   = lPopulate
        .lUseTblBuff = lUseTblBuf
      ENDWITH
    ENDIF
    *** Try and Open the cursor
    llRetVal = This.ProcessRequest( 'Open', loCur )
    RETURN llRetVal	
	ENDFUNC

  *******************************************************************
  *** [P] SETUPOBJECT(): Template Hook Method to initialise the entity
  *** What goes here will depend on the object being created
  *** Called from InitObj()
  *******************************************************************
  PROTECTED FUNCTION SetUpObject( tuSetUpParams )
  
  ENDFUNC

ENDDEFINE

***********************************************************************
*** Root Class Definitions
***********************************************************************
DEFINE CLASS xObjBase AS xCus
  *** Set Flag when Error Message File is in use
  lErrorMsg = .F.
  *** And an Error Collection
  DIMENSION aErrors[1] 

  *******************************************************************
  *** [P] ADDERRORS(): Calls the GetErrors() method of the object passed 
  *** and adds its Error collection to that of the caller 
  *******************************************************************
  PROTECTED FUNCTION AddErrors( toObject )
    LOCAL loErrObj, lnErrs
    *** Check Parameters
    IF VARTYPE( toObject ) # 'O'
      ASSERT .F. MESSAGE 'This is another Brain Dead Programmer Error!' + CHR( 13 ) +;
        'You MUST pass an object reference to the AddErrors Method!' + CHR( 13 ) +;
        'Have a nice day now...'
      RETURN .F.
    ENDIF
    *** Call the specified object's GetErrors() method
    loErrObj = toObject.GetErrors()
    WITH This
      *** Has it got any errors?
      lnErrs = loErrObj.nErrorCount
      IF lnErrs > 0
        *** If so, add them to the current Collection
        FOR lnCnt = 1 TO lnErrs
          .LogError( loErrObj.aErrors[lnCnt, 1], ;
              loErrObj.aErrors[lnCnt, 2], ;
              loErrObj.aErrors[lnCnt, 3] )
        ENDFOR
      ENDIF
    ENDWITH
    RETURN
  ENDFUNC

  *******************************************************************
  *** ERROR(): Define the Error Method
  *** Add the error to the internal errors array
  *******************************************************************
  FUNCTION Error( nError, cMethod, nLine )
    *** Log the error
    This.LogError( nError, MESSAGE(), cMethod )
    RETURN
  ENDFUNC
  
  *******************************************************************
  *** GETERRORS(): Extracts errors from the aErrors collection, 
  *** packages them up in an object, and returns it
  *******************************************************************
  FUNCTION GetErrors 
    LOCAL lnErrCnt, loParamObj, lcErrName, lcErrDets, lnCnt
    *** Create the parameter object
    loParamObj = CREATEOBJECT( 'relation' )
    *** Check if we have any errors
    lnErrCnt = ALEN( This.AErrors, 1 )
    IF lnErrCnt = 1 AND EMPTY( This.AErrors[1,1] )
      *** No Errors Pending
      loParamObj.AddProperty( 'nErrorCount', 0 )
    ELSE
      *** Populate the Parameter Object here
      WITH loParamObj
        .AddProperty( 'nErrorCount', lnErrCnt )
        .AddProperty( 'aErrors[1]', "" )
        *** Now Get the 5 Logged Details
        FOR lnCnt = 1 TO lnErrCnt
          *** Add a row to the error array
          DIMENSION .aErrors[lnCnt, 3]
          *** Log the error as Character Strings
          .aErrors[lnCnt, 1] = TRANSFORM( This.AErrors[ lnCnt , 1] )  && Error Number
          .aErrors[lnCnt, 2] = TRANSFORM( This.AErrors[ lnCnt , 2] )  && Error Message
          .aErrors[lnCnt, 3] = TRANSFORM( This.AErrors[ lnCnt , 3] )  && Prog/Method
        NEXT
      ENDWITH
    ENDIF
    *** Clear the Error Collection
    DIMENSION This.AErrors[1,3]
    This.AErrors = ""
    *** Return Errors
    RETURN loParamObj
  ENDFUNC

  *******************************************************************
  *** [P] GETITEM(): Protected Method to Extract the specified element 
  *** from a separated list Returns NULL when end of list is reached
  *******************************************************************
  PROTECTED FUNCTION GetItem( tcList, tnItem, tcSepBy )
    LOCAL luRetVal, lcSep, lnItem, lnCnt
    luRetVal = ""
    *** Default to Comma Separator if none specified
    lcSep = IIF( VARTYPE( tcSepBy ) # 'C' OR EMPTY( tcSepBy ), ',', tcSepBy )
    *** Default to First Item if nothing specified
    lnItem = IIF( TYPE( 'tnItem' ) # "N" OR EMPTY( tnItem ), 1, tnItem)
    lnCnt = GETWORDCOUNT( tcList, lcSep )
    IF lnItem <= lnCnt
      luRetVal = GETWORDNUM( tcList, lnItem, lcSep )
    ELSE
      luRetVal = NULL
    ENDIF
    *** Return result (Yes, you CAN ALLTRIM a NULL!)
    RETURN ALLTRIM(luRetVal)
  ENDFUNC

  ********************************************************************
  *** [E] INIT(): Standard Initialization Method
  ********************************************************************
  FUNCTION Init()
    *** Do we have an Error Message Table available
    This.lErrorMsg = This.OpenLocalTable("ErrorMsg")
  ENDFUNC

  *******************************************************************
  *** [P] LOGERROR(): Standard Log Error Method
  *** Called from the Error method...
  *** Adds the error to its internal errors collection 
  *******************************************************************
  PROTECTED FUNCTION LogError( tnErrNum, tcErrMsg, tcMethod )
    LOCAL lnSelect, lnErrCnt, lcText, lcErrNum
    lcErrNum = TRANSFORM( tnErrNum )
    lcText = ""
    WITH This
      *** How many Errors currently?
      lnErrCnt = ALEN( .AErrors, 1 )
      *** If only one row, is it actually an error?
      IF lnErrCnt = 1 AND EMPTY( .Aerrors[ 1, 1 ] )
        *** Nope - set error count = 1
        lnErrCnt = 1
      ELSE
        *** We already have at least one error,
        *** so increment the counter
        lnErrCnt = lnErrCnt + 1
      ENDIF
      *** Re-Dimension the Collection
      DIMENSION .AErrors[ lnErrCnt, 3 ]
      *** And Populate the current row
      .AErrors[ lnErrCnt, 1 ] = lcErrNum
      IF This.lErrorMsg 
        IF SEEK( lcErrNum, "ErrorMsg", "cErrNum" ) 
          *** Get the Text fromt the file
          lcText = ALLTRIM( ErrorMsg.cErrText )
        ENDIF
      ENDIF
      .AErrors[ lnErrCnt, 2 ] = lcText + IIF( EMPTY( tcErrMsg) , "", " " + ALLTRIM( tcErrMsg ) )
      .AErrors[ lnErrCnt, 3 ] = IIF( EMPTY( tcMethod) , "", ALLTRIM( tcMethod ) )
    ENDWITH
    *** Just Return Success!
    RETURN .T.
  ENDFUNC

  *******************************************************************
  *** [P] OPENLOCALTABLE(): Opens the specified local VFP Table
  *******************************************************************
  PROTECTED FUNCTION OpenLocalTable( tcTable, tnMode )
    LOCAL lcTable, lcAlias, llRetVal, lcOldErr
    *** Check we got a valid character string
    IF ! This.ValidateParam( tcTable, 'C' )
      RETURN .F.
    ENDIF
    *** Get the table and alias names
    lcTable = JUSTFNAME( tcTable )
    lcAlias = JUSTSTEM( lcTable )
    *** Disable Error Handling
    lcOldErr = ON( "ERROR" )
    ON ERROR *
    *** Check that we have the tables open
    IF ! USED( lcAlias )
      USE ( lcTable ) IN 0 AGAIN SHARED
      llRetVal = USED( lcAlias )
    ELSE
      *** Table is available
      llRetVal = .T.
    ENDIF
    IF llRetVal AND VARTYPE( tnMode ) = 'N'
      *** Did we get a Buffer Mode?
      IF BETWEEN( tnMode, 1, 5 )
        *** Set Buffering on the table to the specified mode
        CURSORSETPROP( 'BUFFERING', tnMode, lcAlias )
      ENDIF
    ENDIF
    *** Log the error if we can't get it
    IF ! llRetVal
      *** Cannot open Source Table
      This.LogError( 9009, tcTable, LOWER( PROGRAM() ) )
    ENDIF
    *** Restore Error Handling
    ON ERROR &lcOldErr
    *** Return Status
    RETURN llRetVal
  ENDFUNC

  *******************************************************************
  *** RELEASE(): Standard Release Method
  *******************************************************************
  PROCEDURE Release
    RELEASE This
  ENDPROC

  *******************************************************************
  *** [P] VALIDATEPARAM(): Check that a value has the specified type
  *** Logs an error and returns .F. if not
  *******************************************************************
  PROTECTED FUNCTION ValidateParam( tuTestParm, tuReqType )
    LOCAL llRetVal, lcCallBy
    llRetVal = .T.
    *** Check for Type Only - program must handle EMPTY() values itself
    IF VARTYPE( tuTestParm ) # UPPER( tuReqType )
      llRetVal = .F.
    ENDIF
    *** Unless it's an object
    IF tuReqType = "O"
      llRetVal = NOT ISNULL( tuTestParm )
    ENDIF
    *** Return Result
    RETURN llRetVal  
  ENDFUNC

ENDDEFINE

***********************************************************************
*** "Buffer" Class Definition
***********************************************************************
DEFINE CLASS xCus AS CUSTOM

  *** Hide unwanted Properties
  PROTECTED Height, HelpContextID, Picture, Readexpression, Readmethod, SaveAsClass 
  PROTECTED ShowWhatsThis, WhatsThisHelp, WhatsThisHelpID, Width, Writeexpression, WriteMethod

ENDDEFINE
