using System;
using System.Windows.Forms;

namespace Westwind.Windows
{
	/// <summary>
	/// Helper class that contains static methods that can be called from overridden controls and forms.
	/// Helpers include Form and Control binding methods.
	/// </summary>
	public class wwDataControlHelper 
	{
		/// <summary>
		/// Returns a property value based on . syntax
		/// </summary>
		/// <param name="loParent">Parent object to 'start' parsing from.</param>
		/// <param name="lcProperty">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyFromString(object loParent, string lcProperty) 
		{
			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0)
				return  loParent.GetType().InvokeMember(lcProperty,
					System.Reflection.BindingFlags.GetField,
					null,loParent,null);

			// *** Walk the . syntax
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);

			object loSub = loParent.GetType().InvokeMember(lcMain,
				System.Reflection.BindingFlags.GetField,
				null,loParent,null);

			// *** Recurse until we get the lowest ref
			return GetPropertyFromString(loSub,lcSubs);
		}


		/// <summary>
		/// Hooks up databinding for all child controls that support it. Will loop through all child containtainers.
		/// </summary>
		/// <param name="loContainer">Form or container object to start parsing from</param>
		public static void FormBindData(Control loContainer) 
		{
			_FormBindData(loContainer,loContainer);
		}

		static void _FormBindData(Control loContainer, Control loMaster)
		{
				foreach (Control control in loContainer.Controls) 
				{
					// ** Recursively call down into containers
					if (control.Controls.Count > 0)
						_FormBindData(control,loMaster);

					if (control is IwwDataControl ) 
					{
						try 
						{
							control.GetType().InvokeMember("BindData",System.Reflection.BindingFlags.InvokeMethod,
								null,control, new object[1] { loMaster } );
						}
						catch(Exception) 
						{
							// *** Display Error info
							try 
							{
								control.Text = "** Field binding Error **";
							}
							catch(Exception) {;} 
						}
					}
				}
			
	
		}


		/// <summary>
		/// Handles control databinding and can be called from the BindData method of subclassed controls
		/// </summary>
		/// <param name="loControl"></param>
		/// <param name="loParent"></param>
		/// <param name="lcControlSourceObject"></param>
		/// <param name="ControlSourceProperty"></param>
		public static void ControlBindData(Control loControl, object loParent,string lcControlSourceObject, 
			string lcControlSourceProperty, string lcBindingProperty)
		{
			if (lcControlSourceObject == null || lcControlSourceObject.Length == 0 ||
				lcControlSourceProperty == null || lcControlSourceProperty.Length == 0)  
				return;

			object loControlSource = Westwind.Tools.wwUtils.GetPropertyExCom(loParent,lcControlSourceObject);

			//			object loControlSource = loParent.GetType().InvokeMember(lcControlSourceObject,
			//				System.Reflection.BindingFlags.GetField,
			//				null,loParent,null);

			// *** can't bind here - object isn't there.
			if (loControlSource == null)
				return;

			//			// *** If we have a datarow - use the property as a field
			//			if ( loControlSource is System.Data.DataRow ) 
			//			{
			//				DataRow loRow = (DataRow) loControlSource;
			//			}


			loControl.DataBindings.Clear();
			loControl.DataBindings.Add(lcBindingProperty,
				loControlSource ,
				lcControlSourceProperty);
			return;
		}

	}
}
