using System;
using System.IO;
using System.Data;
using System.Data.SqlClient;
using System.Xml;
using System.Reflection;

// *** Application specific namespaces
using Westwind.Tools;
using DotNetCOM.WebStoreAdmin;
using DotNetCOM.WebStoreConsumer;

// *** Namespace you'll need for COM Interop
using System.Runtime.InteropServices;

using System.Threading;


namespace DotNetCOM
{
	/// <summary>
	/// DotNetCom sample COM exposed COM object
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("DotNetCom.DotNetComPublisher")]	
	public class DotNetComPublisher
	{
		#region Server Properties
		public string cConnectionString =  "server=(local);database=Pubs;trusted_connection=true"; 
		public string NorthwindConnectionString = "server=(local);database=Northwind;trusted_connection=true"; 
	
		protected string cName = "";
		public string Name 
		{
			get { return this.cName; }
			set { this.cName = value; }
		}
		public string cErrorMsg = "";
		public string ErrorMsg
		{
			get {return this.cErrorMsg; }
			set { this.cErrorMsg = value; }
		}
		protected bool bError = false;
		public bool Error
		{
			get {return this.bError; }
			set { this.bError = value; }
		}

		public DataSet oDs;
		#endregion
	
		public string Test;


		public DotNetComPublisher()
		{
		}

		/// <summary>
		/// Your standard HelloWorld test method.
		/// </summary>
		/// <param name="Name">Name of the user</param>
		/// <returns></returns>
		public string HelloWorld(string Name) 
		{
			return "Hello " + Name + " at " + DateTime.Now.ToString();
		}

		public DateTime GetServerTime() 
		{
			return DateTime.Now;
		}

		public decimal AddNumbers(int lnNumber1, int lnNumber2) 
		{
			return lnNumber1 + lnNumber2;	
		}


#if true
		
		public System.Data.DataSet GetAuthorData(string lcName)
		{
			if (lcName == "" )
				lcName = "%";

			DataSet ds = new DataSet();

			string cConnection = this.cConnectionString;
			SqlConnection	oConn = new SqlConnection(cConnection);
			
			SqlDataAdapter oAdapter = new SqlDataAdapter();

			// *** Make sure we do this to get schema information
			oAdapter.MissingSchemaAction = MissingSchemaAction.AddWithKey;

			oAdapter.SelectCommand = 
				new SqlCommand("select * from Authors where au_lname like '" + lcName + "%' ORDER BY au_lname",oConn);
	            
			try 
			{
				oConn.Open();
			}
			catch(Exception e) 
			{
				this.cErrorMsg = e.Message;
				this.bError = true;
				return null;
			}

			try 
			{
				oAdapter.Fill(ds,"Authors");
			}
			catch (Exception e) 
			{
				this.cErrorMsg = e.Message;
				this.bError = true;
				return null;
			}
			
			oConn.Close();

			this.oDs = ds;	

			return ds;
		}

		public System.Data.DataSet GetAuthorData() 
		{
			return this.GetAuthorData("");
		}


		public string GetAuthorDataXml(string lcName) 
		{
			DataSet loDs = this.GetAuthorData(lcName);
			if (loDs == null)
				return "";
			
			StringWriter loWriter = new StringWriter();
			loDs.WriteXml(loWriter,XmlWriteMode.WriteSchema);
			return loWriter.ToString();
		}


		/// <summary>
		/// Updates the Autors table from an Authors
		/// Table in a DataSet
		/// </summary>
		/// <param name="loDs"></param>
		/// <returns></returns>
		public bool UpdateAuthorData(DataSet loDs)
		{
			this.SetError("");

			string cConnection = this.cConnectionString;
			SqlConnection	oConn = new SqlConnection(cConnection);
			
			SqlDataAdapter oAdapter = new SqlDataAdapter();
			oAdapter.SelectCommand = 
				new SqlCommand("select * from Authors",oConn);

			try 
			{
				// *** Create the Update/Insert Commands
				SqlCommandBuilder oCmdBuilder = new SqlCommandBuilder(oAdapter);
				oAdapter.Update(loDs,"Authors");
			}
			catch (Exception ex)
			{
				this.SetError(ex.Message);
				return false;
			}

			return true;
		}

		
		/// <summary>
		/// Updates the Dataset with an XML Diffgram
		/// </summary>
		/// <param name="lcXml">Diffgram of all changes made from VFP Code</param>
		/// <returns></returns>
		public bool UpdateAuthorDataXml(string lcXml) 
		{
			this.SetError("");

			DataSet loDs = new DataSet();

			loDs.ReadXml(new StringReader(lcXml),XmlReadMode.DiffGram);

			return this.UpdateAuthorData(loDs);
		}



		public byte[] GetBinaryFile(string lcFile) 
		{
			byte[] lcResult = null;

			try 
			{
				FileStream loStream = new FileStream(lcFile,FileMode.Open);
				lcResult = new byte[loStream.Length];
				loStream.Read(lcResult,0,(int) loStream.Length);
				loStream.Close();
			}
			catch(Exception ex) 
			{
				System.Diagnostics.Debug.Assert(false,ex.Message);
				return null;
			}
			return lcResult;
		}

		public string[] GetStrings() 
		{
			string[] strings = new string[3];
			strings[1] = "Rick";
			strings[2] = "Strahl";
			strings[0] = "Test";

			return strings;
		}


		/// <summary>
		/// Pass a VFP style record. Field must have a "Company" field
		/// </summary>
		/// <param name="loObject"></param>
		/// <returns></returns>
		public string PassRecordObject(object FoxObject) 
		{
			// *** using raw Reflection
			string Company = (string) FoxObject.GetType().InvokeMember(
				"Company",   
				BindingFlags.GetProperty,null, 
				FoxObject,null);

			// *** using the easier ComUtils class
			string Name = (string) ComUtils.GetProperty(FoxObject,"Name");

			// *** using ComUtils 'ex' functions to use . Syntax
			string Address = (string) ComUtils.GetPropertyEx(FoxObject,"oAddress.StreetAddress");

			return  Name + Environment.NewLine +
				Company + Environment.NewLine +
				Address + Environment.NewLine + " FOX";
		}

		public string PassCustomerObject(Customer Cust)
		{
			return Cust.Name + Environment.NewLine +
				Cust.Company + Environment.NewLine + 
				Cust.oAddress.StreetAddress + " COM";
		}


		/// <summary>
		/// Return a simple 'static' Customer Object.
		/// </summary>
		/// <returns></returns>
		public Customer ReturnCustomer()
		{
			Customer loCust = new Customer();

			loCust.Name = "Rick Strahl";
			loCust.Company = "West Wind";
			loCust.oAddress.StreetAddress = "32 Kaiea Place\r\nPaia, HI 96779";
			loCust.oAddress.Phone = "808 579-1234";
			loCust.CreditLimit = (decimal) 10000.50;

			return loCust;
		}


		//		/// <summary>
		//		/// Returns a customer object based on a company name.
		//		/// </summary>
		//		/// <param name="lcCompany"></param>
		//		/// <returns></returns>
		//		public Customer GetCustomerObject(string lcCompany)
		//		{
		//			DataSet ds = new DataSet();
		//
		//			string cConnection = this.cConnectionString;
		//			SqlConnection	oConn = new SqlConnection(cConnection);
		//			
		//			SqlDataAdapter oAdapter = new SqlDataAdapter();
		//			oAdapter.SelectCommand = 
		//				new SqlCommand("select * from wws_customers where company like '" + lcCompany + "%'",oConn);
		//	            
		//			try 
		//			{
		//				oConn.Open();
		//			}
		//			catch(Exception e) 
		//			{
		//				this.cErrorMsg = e.Message;
		//				this.bError = true;
		//				return null;
		//			}
		//
		//			try 
		//			{
		//				oAdapter.Fill(ds,"Customer");
		//			}
		//			catch (Exception e) 
		//			{
		//				this.cErrorMsg = e.Message;
		//				this.bError = true;
		//				return null;
		//			}
		//			
		//			oConn.Close();
		//
		//			if (ds.Tables["Customer"].Rows.Count < 1 )
		//			{
		//				return null;
		//			}
		//
		//			Customer loCust = new Customer();
		//			DataRow loRow = ds.Tables["Customer"].Rows[0];
		//
		//			loCust.Name = loRow["FirstName"].ToString().Trim() + " " + loRow["LastName"].ToString();
		//			loCust.Company = loRow["Company"].ToString();
		//			loCust.oAddress.StreetAddress = loRow["Address"].ToString();
		//			loCust.oAddress.Phone = loRow["Phone"].ToString();
		//			loCust.oAddress.Email = loRow["Email"].ToString();
		//			loCust.Entered = (DateTime) loRow["Entered"];
		//		
		//			return loCust;
		//		}


		private void SetError(string lcError) 
		{
			if (lcError != null && lcError.Length > 0) 
			{
				this.cErrorMsg = lcError;
				this.bError = true;
			}
			else 
			{
				this.cErrorMsg = "";
				this.bError = false;
			}
		}

		public byte[] ReturnBytes() 
		{
			return Guid.NewGuid().ToByteArray();
		}

		public Guid ReturnGuid() 
		{
			return Guid.NewGuid();
		}

		/// <summary>
		/// Passing an array from FoxPro to .NET
		/// Note that any FoxPro objects will require REFLECTION
		/// to be used!!!
		/// </summary>
		/// <param name="Objects"></param>
		/// <returns></returns>
		public string PassArray(object[] Objects ) 
		{
			string Output = "";

			foreach(object Customer in Objects) 
			{
				// *** Must access array by using Reflection
				Output += ComUtils.GetProperty(Customer,"Name") + " " + 	
					ComUtils.GetProperty(Customer,"Company") + "\r\n";

			}

			return Output;
		}

		public Customer[] ReturnArray(int Count) 
		{
			Customer[] Customers = new Customer[Count];
			for (int x=0; x < Count; x++)
			{
				Customers[x] = new Customer();
				Customers[x].Company = "West Wind " + DateTime.Now.ToString();
				Customers[x].oAddress.StreetAddress = DateTime.Now.ToString();
			}

			return Customers;
		}


		public Assembly LoadAssembly(string Filename)
		{
			Assembly Ass;
			try 
			{
				Ass = Assembly.LoadFrom(Filename);
			}
			catch(Exception ex)
			{
				this.cErrorMsg = ex.Message;
				return null;
			}

			return Ass;
		}

		/// <summary>
		/// Retrieve an array of invoice entities from a Web service.
		/// </summary>
		/// <returns></returns>
		public WebStoreAdmin.InvoiceEntity[] 
			DownloadWestWindOrders(string Url,string Username,string Password) 
		{
			WebStoreAdminService Service = this.LoadAdminWebService(Url,Username,Password);
			WebStoreAdmin.InvoiceEntity[] Invoices = Service.GetNonDownloadedInvoiceEntities(false);

			return Invoices;
		}


		/// <summary>
		/// Upload an individual order to the Web Store site
		/// ... problematic - Fox can't resize array passed from .NET
		/// </summary>
		/// <param name="Invoice"></param>
		/// <returns></returns>
		public bool UploadWestWindOrder(WebStoreConsumer.InvoiceEntity Invoice) 
		{
			WebStoreService Service = this.LoadConsumerWebService(null,null,null);
			return Service.UploadInvoiceEntity(Invoice);
		}

		/// <summary>
		/// Returns an InvoiceEntity for the Web Store Inovice Upload
		/// </summary>
		/// <returns></returns>
		public WebStoreConsumer.InvoiceEntity GetInvoiceEntity() 
		{
			WebStoreService Service = this.LoadConsumerWebService(null,null,null);
			return Service.GetInvoiceEntity();
		}


		/// <summary>
		/// Loader method for the Web Service so we have all the configuration
		/// settings in one place
		/// </summary>
		/// <returns></returns>
		private WebStoreAdmin.WebStoreAdminService LoadAdminWebService() 
		{
			return this.LoadAdminWebService(null,null,null);
		}

		/// <summary>
		/// Loader method for the Web Service so we have all the configuration
		/// settings in one place
		/// </summary>
		private WebStoreAdmin.WebStoreAdminService LoadAdminWebService(string Url,string Username, string Password) 
		{
			WebStoreAdmin.WebStoreAdminService Service = new DotNetCOM.WebStoreAdmin.WebStoreAdminService();
			
			// *** Use the default proxy - IE configurations
			Service.Proxy = System.Net.WebProxy.GetDefaultProxy();
			
			if (Url != null && Url != String.Empty)
				Service.Url = Url;
			if (Username != null && Url !=String.Empty) 
				Service.Credentials = new System.Net.NetworkCredential(Username,Password);
			else 
			{ 
				// Use Your Windows Login - not working here because ASP.NET Auth
				//Service.Credentials = System.Net.CredentialCache.DefaultCredentials;	
				
				// *** Use 'demo' username and password
				Service.Credentials = new System.Net.NetworkCredential(
					wwUtils.GetSystemPassword(true),wwUtils.GetSystemPassword(false));
			}

			return Service;
		}

		/// <summary>
		/// Method that loads up the Consumer web Service
		/// </summary>
		/// <param name="Url"></param>
		/// <param name="Username"></param>
		/// <param name="Password"></param>
		/// <returns></returns>
		private WebStoreConsumer.WebStoreService LoadConsumerWebService(string Url,string Username, string Password) 
		{
			WebStoreConsumer.WebStoreService Service = new WebStoreConsumer.WebStoreService();
			if (Url != null)
				Service.Url = Url;
			if (Username != null) 
				Service.Credentials = new System.Net.NetworkCredential(Username,Password);
			else 
				// *** Use demo passwords <g> - retrieved from AutoLogon section in
				// *** the registry
				Service.Credentials = new System.Net.NetworkCredential(
					wwUtils.GetSystemPassword(true),wwUtils.GetSystemPassword(false));

			return Service;
		}

		

	
		//		public string PassFoxCOMObject( tlibtest.CustomerClass Customer) 
		//		{
		//			tlibtest.Orders Order = Customer.GetOrder("1");
		//			return Order.ORDER_ID.ToString();
		//		}

		public void Unload() 
		{
			
		}
		
#endif
	

	
	}



}