using System;
using Microsoft.Win32;
using System.Runtime.InteropServices;

namespace Westwind.WebMonitor.Components
{
	/// <summary>
	/// Summary description for WebMonitorConfig.
	/// </summary>
	/// <summary>
	/// Class that contains the global settings for a Web Monitor
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("WebMonitor.WebMonitorConfig")]
	public class WebMonitorConfig : MarshalByRefObject
	{
		public decimal Version = 3.30M;
		/// <summary>
		/// Default mail Server used to send email alerts.
		/// </summary>
		public string MailServer
		{
			get { return this.cMailServer; }
			set { this.cMailServer = value; }
		}
		string cMailServer = "";
	
		public string MailServerUsername 
		{
			get { return this.cMailServerUsername; }
			set { this.cMailServerUsername = value; }
		}
		string cMailServerUsername = "";

		public string MailServerPassword
		{
			get { return this.cMailServerPassword; }
			set { this.cMailServerPassword = value; }
		}
		string cMailServerPassword = "";

		/// <summary>
		/// Email address used as the return address when error emails are sent. This is the default and it's overridden by the Site
		/// </summary>
		public string EmailFrom
		{
			get { return this.cEmailFrom; }
			set { this.cEmailFrom = value; }
		}
		string cEmailFrom = "";

		/// <summary>
		/// The default recipients for email notifications. Loaded as the default value.
		/// </summary>
		public string Recipients
		{
			get { return this.cRecipients; }
			set { this.cRecipients = value; }
		}
		string cRecipients = "";

		string cProxyServer = "";
		public string ProxyServer 
		{
			get {return cProxyServer; }
			set {this.cProxyServer = value;}
		}

		string cProxyByPass = "" ;
		public string ProxyByPass
		{
			get {return cProxyByPass; }
			set {this.cProxyByPass = value;}
		}
		
		string cProxyUsername = "";
		public string ProxyUsername
		{
			get {return cProxyUsername; }
			set {this.cProxyUsername = value;}
		}

		string cProxyPassword = "";
		public string ProxyPassword
		{
			get {return cProxyPassword; }
			set {this.cProxyPassword = value;}
		}

		/// <summary>
		/// The Web Monitor Error log. Only holds failed requests. Unlike
		/// the detail log this log is always written.
		/// </summary>
		public string LogFile 
		{
			get { return this.cLogFile; }
			set { this.cLogFile = value; }
		}
		string cLogFile = "WebMonitorErrorLog.xml";

		/// <summary>
		/// File that receives each log request and result entry. Basically
		/// picks up the events that are usually sent to the UI and logs
		/// them into a log file so they can be read externally.
		/// </summary>
		public string DetailLogFile
		{
			get { return this.cDetailLogFile; }
			set { this.cDetailLogFile = value; }
		}
		string cDetailLogFile = "WebMonitorDetailLog.xml";

		/// <summary>
		/// Used to determine how big the detail log file gets (in bytes). The file is truncated from the
		/// beginning. A value of 0 means no logging is to occur. Default size is 1meg.
		/// </summary>
		public int DetailLogFileSize
		{
			get { return this.nDetailLogFileSize; }
			set 
			{ 
				if (value > 0 && value < 100000)
					this.nDetailLogFileSize = 100000;	
				else	
					this.nDetailLogFileSize = value;
			}
		}
		int nDetailLogFileSize = 200000;


		/// <summary>
		/// Detemines how Web Monitor starts. Options allow for startup on the TaskBar or as a
		/// Windows Service or just as a normal Windows app. Taskbar operation is just like 
		/// standalone, but starts invisible.
		/// </summary>
		public WebMonitorAutoLaunchMode LaunchOnStartup 
		{
			get { return this.bLaunchOnStartup; }
			set { this.bLaunchOnStartup = value; }
		}
		WebMonitorAutoLaunchMode bLaunchOnStartup;

	}


	public enum WebMonitorAutoLaunchMode 
	{
		NoAutoLaunch,
		AutoLaunchOnTaskBar,
		LaunchAsService
	}

}
