using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Reflection;

using Microsoft.Win32;


namespace Westwind.WebMonitor.Components
{
	/// <summary>
	/// Summary description for wwUtils.
	/// </summary>
	public class wwUtils
	{
			
		/// <summary>
		/// Fixes a plain text field for display as HTML by replacing carriage returns 
		/// with the appropriate br and p tags for breaks.
		/// </summary>
		/// <param name="String Text">Input string</param>
		/// <returns>Fixed up string</returns>
		public static string DisplayMemo(string HtmlText) 
		{				
			HtmlText = HtmlText.Replace("\r\n","\r");
			HtmlText = HtmlText.Replace("\n","\r");
			HtmlText = HtmlText.Replace("\r\r","<p>");
			HtmlText = HtmlText.Replace("\r","<br>");
			return HtmlText;
		}

		/// <summary>
		/// Method that handles handles display of text by breaking text.
		/// Unlike the non-encoded version it encodes any embedded HTML text
		/// </summary>
		/// <param name="Text"></param>
		/// <returns></returns>
		public static string DisplayMemoEncoded(string Text) 
		{
			bool PreTag = false;
			if (Text.IndexOf("<pre>") > -1) 
			{
				Text = Text.Replace("<pre>","__pre__");
				Text = Text.Replace("</pre>","__/pre__");
				PreTag = true;
			}

			// *** fix up line breaks into <br><p>
			Text = wwUtils.DisplayMemo( System.Web.HttpUtility.HtmlEncode(Text) );

			if (PreTag) 
			{
				Text = Text.Replace("__pre__","<pre>");
				Text = Text.Replace("__/pre__","</pre>");
			}

			return Text;
		}


		[DllImport("Shell32.dll")]
		private static extern int ShellExecute(int hwnd, string lpOperation, 
			string lpFile, string lpParameters, 
			string lpDirectory, int nShowCmd);

		[DllImport("user32.dll")]
		public static extern int SetForegroundWindow(int hwnd);

		/// <summary>
		/// Uses the Shell Extensions to launch a program based or URL moniker.
		/// </summary>
		/// <param name="lcUrl">Any URL Moniker that the Windows Shell understands (URL, Word Docs, PDF, Email links etc.)</param>
		/// <returns></returns>
		public static int GoUrl(string lcUrl)
		{
			string lcTPath = Path.GetTempPath();

			string Username = Environment.UserName;
			

			int lnResult = ShellExecute(0,"OPEN",lcUrl, "",lcTPath,1);
			return lnResult;
		}


		/// <summary>
		/// Returns the physical location of a given assembly. 
		/// </summary>
		/// <param name="loStartAssemblyObject">An object that is located in the startup assembly. If you leave this out the value is set to wwUtils (this class) which may or may not be located in the startup assembly.</param>
		/// <returns></returns>
		public static string GetAppStartPath(object loStartAssemblyObject) 
		{
			if (loStartAssemblyObject == null) 
				loStartAssemblyObject = new wwUtils();

			string lcStartupPath = loStartAssemblyObject.GetType().Assembly.Location;
			FileInfo loFile = new FileInfo(lcStartupPath);
			return loFile.DirectoryName;
		}

		/// <summary>
		/// Sets the Working Set (Memory Usage) of an application to the given size of bytes. 
		/// Will try to 'squeeze' process memory to the specified size by releasing unused code,
		/// specifically code for the JIT compiler. 
		/// 
		/// Recommended use is to call this method after all your code has been compiled to
		/// release the JIT compiler.
		/// </summary>
		/// <param name="lnMaxSize">Maximum memory used</param>
		/// <param name="lnMinSize">Minimum Memory used</param>
		public static void SetWorkingSet(int lnMaxSize,int lnMinSize) 
		{
			try  
			{
				System.Diagnostics.Process loProcess = 	System.Diagnostics.Process.GetCurrentProcess();
				loProcess.MaxWorkingSet = (IntPtr) lnMaxSize;
				loProcess.MinWorkingSet = (IntPtr) lnMinSize;
				long lnValue = loProcess.WorkingSet;
			}
			catch
			{ 
				/// Ignore the error - we can have security issues with this one
			}

			
		}


		/// <summary>
		/// Retrieve a dynamic 'non-typelib' property
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="lcProperty"></param>
		/// <returns></returns>
		public static object GetProperty(object loObject,string lcProperty)
		{
			return loObject.GetType().InvokeMember(lcProperty,BindingFlags.GetProperty,null,loObject,null);
		}

		/// <summary>
		/// Returns a property value using a base object and sub members including . syntax.
		/// For example, you can access: this.oCustomer.oData.Company with (this,"oCustomer.oData.Company")
		/// </summary>
		/// <param name="loParent">Parent object to 'start' parsing from.</param>
		/// <param name="lcProperty">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyEx(object loParent, string lcProperty) 
		{
			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0)
				return  loParent.GetType().InvokeMember(lcProperty,
					System.Reflection.BindingFlags.GetProperty,
					null,loParent,null);

			// *** Walk the . syntax
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);

			object loSub = loParent.GetType().InvokeMember(lcMain,
				System.Reflection.BindingFlags.GetProperty,
				null,loParent,null);

			// *** Recurse until we get the lowest ref
			return GetPropertyEx(loSub,lcSubs);
		}

		/// <summary>
		/// Set Property dynamically
		/// </summary>
		/// <param name="loObject">The object to set this property on.</param>
		/// <param name="lcProperty">The name of the property to set</param>
		/// <param name="loValue">The value to set it to</param>
		/// <returns>nothing</returns>
		public static void SetProperty(object loObject,string lcProperty,params object[] loValue)
		{
			loObject.GetType().InvokeMember(lcProperty,BindingFlags.SetProperty,null,loObject,loValue);
		}

		/// <summary>
		/// Sets a property using . Syntax
		/// </summary>
		/// <param name="loParent">Object to start from</param>
		/// <param name="lcProperty">. Syntax property value (oData.Company)</param>
		/// <param name="loValue">Value to set it to</param>
		/// <returns>used internally only for recursive calls. You can ignore the result value</returns>
		public static object SetPropertyEx(object loParent, string lcProperty,params object[] loValue) 
		{
			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0)
				// *** no more .s - we got our final object
				return  loParent.GetType().InvokeMember(lcProperty,
					System.Reflection.BindingFlags.SetProperty,
					null,loParent,loValue);

			// *** Walk the . syntax
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);

			object loSub = loParent.GetType().InvokeMember(lcMain,
				System.Reflection.BindingFlags.GetProperty,
				null,loParent,null);

			// *** Recurse until we get the lowest ref
			return SetPropertyEx(loSub,lcSubs,loValue);
		}

		/// <summary>
		/// Wrapper method to call a 'dynamic' (non-typelib) method
		/// on a COM object
		/// </summary>
		/// <param name="loParams"></param>
		/// 1st - Method name, 2nd - 1st parameter, 3rd - 2nd parm etc.
		/// <returns></returns>
		public static object CallMethod(object loObject,string lcMethod, params object[] loParams)
		{
			return loObject.GetType().InvokeMember(lcMethod,
				BindingFlags.InvokeMethod,null,loObject,loParams);
		}

		public static void OpenHelpBuilder(ref object loHelp,string lcHelpId) 
		{
			if (loHelp == null) 
			{
				Type loT = Type.GetTypeFromProgID("wwHelp.wwHelp");
				loHelp =  Activator.CreateInstance(loT);
			}

			if (lcHelpId != null && lcHelpId.Length > 0) 
			{
				int lnIndex = lcHelpId.LastIndexOf(".");
				if (lnIndex > 0)
					lcHelpId = lcHelpId.Substring(0,lcHelpId.LastIndexOf("."));

				wwUtils.CallMethod(loHelp,"OpenForm",lcHelpId,0);
			}
		}


	}

	
}