using System;
using System.Reflection;
using System.Runtime.InteropServices;

namespace Westwind.wwReflection
{
/// <summary>
/// This class manages executing a .NET method in a .NET assembly
/// </summary>
[ClassInterface(ClassInterfaceType.AutoDual)]
[ProgId("wwReflection.AddinExecution")]
public class AddinExecution : MarshalByRefObject
{
	public string ErrorMessage = "";
	public bool Error = false;

	public AddinExecution()
	{
	}

	/// <summary>
	//	/ The Actual worker method that dynamically invokes the assembly, type and method.
	/// </summary>
	/// <param name="AssemblyFile"></param>
	/// <param name="Typename"></param>
	/// <param name="Method"></param>
	/// <param name="Parameter"></param>
	/// <returns></returns>
	public bool Execute(string AssemblyFile, string Typename, string Method, object Parameter) 
	{
		Assembly ass = null;
		try 
		{
			ass = Assembly.LoadFrom(AssemblyFile);
		}
		catch(Exception ex)
		{
			this.Error = true;
			this.ErrorMessage = ex.Message;
			return false;
		}
		
		Type TypeRef = null;
		try
		{
			TypeRef	= ass.GetType(Typename,true,true);
		}
		catch(Exception ex) 
		{
			this.ErrorMessage = ex.Message;
			return false;
		}

		
		try 
		{
			object Instance = Activator.CreateInstance(TypeRef);
		
			// *** Convert object into strongly typed object
			Westwind.wwHelp.wwHelpForm Form = new Westwind.wwHelp.wwHelpForm(Parameter);

			bool Result = (bool) Instance.GetType().InvokeMember(Method,BindingFlags.InvokeMethod | BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic,null,
				Instance,new object[1] { Form } );
			return Result;
		}
		catch(Exception ex) 
		{
			this.Error = true;
			this.ErrorMessage = ex.Message;
			return false;
		}

		return true;
	}
}


[ClassInterface(ClassInterfaceType.AutoDual)]
[ProgId("wwReflection.AddinFactory")]
public class AddinFactory : MarshalByRefObject 
{
	/// <summary>
	/// Reference to the AppDomain that the TypeParser
	/// is loaded into.
	/// </summary>
	AppDomain LocalAppDomain = null;

	/// <summary>
	/// TypeParser Factory method that loads the TypeParser
	/// object into a new AppDomain so it can be unloaded.
	/// Creates AppDomain and creates type.
	/// </summary>
	/// <returns></returns>
	public object CreateAddin() 
	{
		if (!CreateAppDomain(null))
			return null;

		// *** Use a custom Assembly Resolver that looks in the CURRENT directory
		//			this.LocalAppDomain.AssemblyResolve += new ResolveEventHandler( TypeParserFactory.ResolveAssembly );

		/// Create the instance inside of the new AppDomain
		/// Note: remote domain uses local EXE's AppBasePath!!!
		object T = null;

		try 
		{
			T = this.LocalAppDomain.CreateInstance("wwReflection",
				"Westwind.wwReflection.AddinExecution").Unwrap();
		}
		catch 
		{
			return null;
		}

			
		if (T == null)
			return null;
		
		return T as AddinExecution;
	}

	private bool CreateAppDomain(string lcAppDomain) 
	{
		if (lcAppDomain == null)
			lcAppDomain = "wwReflection" + Guid.NewGuid().ToString().GetHashCode().ToString("x");

		AppDomainSetup loSetup = new AppDomainSetup();

		// *** Point at current directory
		loSetup.ApplicationBase = AppDomain.CurrentDomain.BaseDirectory;

		this.LocalAppDomain = AppDomain.CreateDomain(lcAppDomain,null,loSetup);
		
		return true;
	}

	public void UnloadAddin() 
	{
		if (this.LocalAppDomain != null) 
		{
			AppDomain.Unload( this.LocalAppDomain );
			this.LocalAppDomain = null;
		}
	}

				
}	


}
