using System;
using System.Runtime.InteropServices;	

using System.Reflection;
using System.Xml;

using  Westwind.wwReflection.Tools;

namespace Westwind.wwReflection
{
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.DotNetObject")]
	[Serializable]
	public class DotnetObject
	{
		public string cName = "";
		public string cScope = "";
		public string cSyntax = "VB";
		
		public ObjectMethod[] aMethods;
		public int nMethodCount = 0;

		public ObjectProperty[] aProperties;
		public int nPropertyCount = 0;

		public ObjectEvent[] aEvents;
		public int nEventCount = 0;
		

		public string cHelpText = "";
		public string cRemarks = "";
		public string cExample = "";
		public string cSeeAlso = "";

		public string cAssembly = "";
		public string cNamespace = "";
		public string cInheritsFrom = "";
		public string cInheritanceTree = "";
		public string cImplements = "";

		/// <summary>
		/// class, interface, enum, delegate
		/// </summary>
		public string cType = "";

		/// <summary>
		/// Full signature of the class - namespace + classname
		/// </summary>
		public string cSignature = "";

		/// <summary>
		/// All class level identifiers except visibility. ie: static override virtual etc.
		/// </summary>
		public string cOther = "";

		/// <summary>
		/// Determines whether inherited members are retrieved
		/// </summary>
		public bool RetrieveDeclaredMembersOnly 
		{
			get { return _RetrieveDeclaredMembersOnly; }
			set 
			{
				if (value)
					this.RetrievalFlags =	BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static;
				else
					this.RetrievalFlags =	BindingFlags.Instance |  BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static;
				
				_RetrieveDeclaredMembersOnly = value;
			}
		}
		bool _RetrieveDeclaredMembersOnly = false;

		/// <summary>
		/// Reflection retrieval flags.
		/// </summary>
		protected BindingFlags RetrievalFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static;

		/// <summary>
		/// Loads all methods for a given type into the aMethod array
		/// </summary>
		/// <param name="loType">the type to load methods from</param>
		/// <returns></returns>
		public int  LoadMethods(Type loType) 
		{
			MethodInfo[] loMethods = loType.GetMethods( this.RetrievalFlags );
			if (loMethods.Length < 1) 
			{
				this.nMethodCount = 0;
				return 0;
			}
			
			ConstructorInfo[] loCtors = loType.GetConstructors(this.RetrievalFlags);

			int lnCount = loMethods.Length;
			this.aMethods = new ObjectMethod[lnCount + loCtors.Length];
			

			for (int x=0 ; x < lnCount; x++) 
			{
				MethodInfo loMethod = loMethods[x];
				this.aMethods[x] = new ObjectMethod();

				// *** Skip Property Get and Set methods -- We must check for shorter props
				///				if (loMethod.Name.Substring(0,4) == "set_" ||
				///					loMethod.Name.Substring(0,4) == "get_")
				///				   continue;

				ObjectMethod loM = this.aMethods[x];
				loM.cName = loMethod.Name;
				if (loM.cName == null) 
					continue;

				this.nMethodCount++;

				
				if (loMethod.IsPublic) 
					loM.cScope = "public";
				else if (loMethod.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				if ( this.cSyntax.StartsWith("VB") ) 
					loM.cScope = wwUtils.ProperCase(loM.cScope);

				if (loMethod.IsStatic) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "Shared|";
					else
						loM.cOther += "static|";
					loM.bStatic = true;
				}
				if (loMethod.IsAbstract) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "MustOverride|";
					else	
						loM.cOther += "abstract|";
				}
				if (loMethod.IsFinal) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "NotOverridable|";
					else
						loM.cOther += "sealed|";
				}
				if (loMethod.IsVirtual && !loMethod.IsAbstract && !loMethod.IsFinal)  // don't want virtual abstract
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther +=  "Overridable|";
					else	
						loM.cOther +=  "virtual|";
				}

				ParameterInfo[] loParameters = loMethod.GetParameters();
				loM.nParameterCount = loParameters.Length;

				if (loParameters.Length > 0) 
					loM.aParameters = new string[loParameters.Length];


				string lcParameters = "";
				string lcRawParameters = "";
				
				for (int y = 0; y < loParameters.Length; y++) 
				{
					if (this.cSyntax == "VB") 
						lcParameters = lcParameters + loParameters[y].Name + " as " + this.TypeNameForLanguage(loParameters[y].ParameterType.Name);
					else
						lcParameters = lcParameters +  this.TypeNameForLanguage(loParameters[y].ParameterType.Name) + " " +loParameters[y].Name;

					if (y < loParameters.Length - 1) 
						lcParameters= lcParameters + ", ";

					lcRawParameters = lcRawParameters + loParameters[y].ParameterType.FullName;
					if (y < loParameters.Length - 1) 
						lcRawParameters= lcRawParameters + ",";

					loM.aParameters[y] = loParameters[y].Name;
				}
				loM.cParameters = lcParameters;
				loM.cRawParameters = lcRawParameters;
				
				loM.cSignature = loMethod.DeclaringType.FullName + "." + loMethod.Name;
				loM.cSignature = loM.cSignature + "(" + lcRawParameters + ")";

				loM.cDeclaringType = loMethod.DeclaringType.FullName;
				
				
				loM.cReturnType = this.TypeNameForLanguage(loMethod.ReturnType.Name);
			}

			
			for (int x = 0; x < loCtors.Length;x++) 
			{
				ConstructorInfo loMethod = loCtors[x];
				this.aMethods[this.nMethodCount] = new ObjectMethod();

				// *** Skip Property Get and Set methods -- We must check for shorter props
				///				if (loMethod.Name.Substring(0,4) == "set_" ||
				///					loMethod.Name.Substring(0,4) == "get_")
				///				   continue;

				ObjectMethod loM = this.aMethods[this.nMethodCount];
				loM.cName = "Constructor"; //" Constructor"; "_" + loMethod.Name.Substring(1);  
				if (loM.cName == null) 
					continue;

				this.nMethodCount++;
				
				if (loMethod.IsPublic) 
					loM.cScope = "public";
				else if (loMethod.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				if ( this.cSyntax.StartsWith("VB") ) 
					loM.cScope = wwUtils.ProperCase(loM.cScope);

				if (loMethod.IsStatic) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "Shared|";
					else
						loM.cOther += "static|";
					loM.bStatic = true;
				}
				if (loMethod.IsAbstract) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "MustOverride|";
					else	
						loM.cOther += "abstract|";
				}
				if (loMethod.IsFinal) 
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "NotOverridable|";
					else
						loM.cOther += "sealed|";
				}
				if (loMethod.IsVirtual && !loMethod.IsAbstract && !loMethod.IsFinal)  // don't want virtual abstract
				{
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther +=  "Overridable|";
					else	
						loM.cOther +=  "virtual|";
				}

				ParameterInfo[] loParameters = loMethod.GetParameters();
				loM.nParameterCount = loParameters.Length;

				if (loParameters.Length > 0) 
					loM.aParameters = new string[loParameters.Length];


				string lcParameters = "";
				string lcRawParameters = "";
				
				for (int y = 0; y < loParameters.Length; y++) 
				{
					if (this.cSyntax == "VB") 
						lcParameters = lcParameters + loParameters[y].Name + " as " + this.TypeNameForLanguage(loParameters[y].ParameterType.Name);
					else
						lcParameters = lcParameters +  this.TypeNameForLanguage(loParameters[y].ParameterType.Name) + " " +loParameters[y].Name;

					if (y < loParameters.Length - 1) 
						lcParameters= lcParameters + ", ";

					lcRawParameters = lcRawParameters + loParameters[y].ParameterType.FullName;
					if (y < loParameters.Length - 1) 
						lcRawParameters= lcRawParameters + ",";

					loM.aParameters[y] = loParameters[y].Name;
				}
				loM.cParameters = lcParameters;
				loM.cRawParameters = lcRawParameters;
				
				loM.cSignature = loMethod.DeclaringType.FullName + ".#" + loMethod.Name.Replace(".","");
				loM.cSignature = loM.cSignature + "(" + lcRawParameters + ")";

				loM.cDeclaringType = loMethod.DeclaringType.FullName;
				
				loM.cReturnType = "";
			}

			return this.nMethodCount;
		}

		/// <summary>
		/// Loads properties and fields into the aProperties array.
		/// </summary>
		/// <param name="loType">the type to load from</param>
		/// <returns></returns>
		public int  LoadProperties(Type loType) 
		{
			FieldInfo[] loFields = loType.GetFields( this.RetrievalFlags );
			PropertyInfo[] loProperties = loType.GetProperties(this.RetrievalFlags);

			int lnCount = loFields.Length + loProperties.Length;
			if (lnCount < 1) 
			{
				return 0;
			}
			this.nPropertyCount = lnCount;
			this.aProperties = new ObjectProperty[lnCount];

			for (int x=0; x < loFields.Length; x++) 
			{
				FieldInfo loProperty = loFields[x];
				this.aProperties[x] = new ObjectProperty();

				ObjectProperty loM = this.aProperties[x];
				loM.cName = loProperty.Name;
				loM.cFieldOrProperty = "Field";
				loM.cType =  this.TypeNameForLanguage(loProperty.FieldType.Name);

				if (loProperty.IsPublic) 
					loM.cScope = "public";
				else if (loProperty.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				if ( this.cSyntax.StartsWith("VB") ) 
					loM.cScope = wwUtils.ProperCase(loM.cScope);
				
				if (loProperty.IsStatic) 
				{
					loM.bStatic = true;
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther = "Shared|";
					else
						loM.cOther = "static|";
				}
				if (loProperty.IsLiteral) 
				{
					loM.bLiteral = true;
				}

				loM.cSignature = loProperty.DeclaringType.FullName + "." + loProperty.Name;
			}

			
			if (loProperties.Length < 1)
				return lnCount;
			
			int lnFieldCount = loFields.Length;
			for (int x=0; x < loProperties.Length; x++) 
			{
				PropertyInfo loProperty = loProperties[x];
				this.aProperties[lnFieldCount + x] = new ObjectProperty();

				ObjectProperty loM = this.aProperties[lnFieldCount + x];
				loM.cName =  loProperty.Name;
				loM.cFieldOrProperty = "Property";
				loM.cType = this.TypeNameForLanguage(loProperty.PropertyType.Name);
				
				
				// *** Must use Accessor method to get additional info
				MethodInfo mi = null;
				if (loProperty.CanRead)
					mi = loProperty.GetGetMethod(true);
				else
					mi = loProperty.GetSetMethod(true);

				if (mi.IsPublic) 
					loM.cScope = "public";
				else if (mi.IsPrivate)
					loM.cScope = "private";
				else 
					loM.cScope = "protected";

				if ( this.cSyntax.StartsWith("VB") ) 
					loM.cScope = wwUtils.ProperCase(loM.cScope);
				
				if (mi.IsStatic) 
				{
					loM.bStatic = true;
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther = "Shared|";
					else
						loM.cOther = "static|";
				}
				if (mi.IsVirtual)
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "Overridable|";
					else
						loM.cOther += "virtual|";
				if (mi.IsAbstract)
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "MustOverride|";
					else
						loM.cOther += "abstract|";
				if (mi.IsFinal)
					if (this.cSyntax.StartsWith("VB")) 
						loM.cOther += "NotOverridable|";
					else
						loM.cOther += "sealed|";
					


				if (loProperty.CanRead && loProperty.CanWrite) 
					loM.cAccessType = "";
				else if(loProperty.CanRead) 
					loM.cAccessType = "ReadOnly";
				else if(loProperty.CanWrite)
					loM.cAccessType = "WriteOnly";

				loM.cSignature = loProperty.DeclaringType.FullName + "." + loProperty.Name;
			}

			return lnCount;
		}


		/// <summary>
		/// Load Events from an object into aEvents property
		/// </summary>
		/// <param name="loType">the type to load from</param>
		/// <returns></returns>
		public int  LoadEvents(Type loType)  
		{
			EventInfo[] loFields = loType.GetEvents( this.RetrievalFlags );
			
			int lnCount = loFields.Length ;
			if (lnCount < 1) 
				return 0;

			this.nEventCount = lnCount;
			this.aEvents = new ObjectEvent[lnCount];

			for (int x=0; x < loFields.Length; x++) 
			{
				EventInfo loEvent= loFields[x];
				this.aEvents[x] = new ObjectEvent();

				ObjectEvent loM = this.aEvents[x];
				loM.cName = loEvent.Name;
				loM.cType = loEvent.EventHandlerType.Name;

				loM.cScope = "public";
				if ( this.cSyntax.StartsWith("VB") ) 
					loM.cScope = wwUtils.ProperCase(loM.cScope);

				loM.cSignature = loEvent.DeclaringType.FullName + "." + loEvent.Name;
			}

			return lnCount;
		}

		
		/// <summary>
		/// Converts a CLR typename to VB or C# type name. Type must passed in as plain name
		/// not in full system format.
		/// </summary>
		/// <param name="Type"></param>
		/// <param name="Language"></param>
		/// <returns></returns>
		public string TypeNameForLanguage(string Type) 
		{
			if (this.cSyntax != "VB")
			{
				switch(Type) 
				{
					case "String":
						return "string";
					case "Boolean":
						return "bool";
					case "Int32":
						return "int";
					case "Int64":
						return "long";
					case "Int16":
						return "byte";
					case "Decimal":
						return "decimal";
					case "Object":
						return "object";
					case "Double":
						return "double";
					case "Single":
						return "float";
					case "Char":
						return "char";
					case "Void":
						return "void";
				}
			}
			else 
			{
				switch(Type)
				{
					case "DateTime":
						return "Date";
					case "Int32":
						return "Integer";
					case "Int64":
						return "Long";
					case "Int16":
						return "Short";
				}
			}
			
			return Type;
		}

	}
}
