using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Collections.Specialized;

using System.IO;
using System.Text;
using System.Reflection;
using System.Xml;

using System.Runtime.Remoting;

using Westwind.wwReflection.Tools;

namespace Westwind.wwReflection
{
	/// <summary>
	/// TypeParserFactory object used to create a TypeParser instance
	/// in another AppDomain. This is done so the classes reflected over
	/// can be unloaded. 
	/// 
	/// Note should use one instance per 
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.TypeParserFactory")]
	public class TypeParserFactory : System.MarshalByRefObject,IDisposable	
	{
		/// <summary>
		/// Reference to the AppDomain that the TypeParser
		/// is loaded into.
		/// </summary>
		AppDomain LocalAppDomain = null;

		/// <summary>
		/// TypeParser Factory method that loads the TypeParser
		/// object into a new AppDomain so it can be unloaded.
		/// Creates AppDomain and creates type.
		/// </summary>
		/// <returns></returns>
		public TypeParser CreateTypeParser() 
		{
			if (!CreateAppDomain(null))
				return null;

			// *** Use a custom Assembly Resolver that looks in the CURRENT directory
			//			this.LocalAppDomain.AssemblyResolve += new ResolveEventHandler( TypeParserFactory.ResolveAssembly );

			/// Create the instance inside of the new AppDomain
			/// Note: remote domain uses local EXE's AppBasePath!!!
			object T = null;

			try 
			{
				T = this.LocalAppDomain.CreateInstance("wwReflection",
					"Westwind.wwReflection.TypeParser").Unwrap();
			}
			catch 
			{
				return null;
			}

			if (T == null)
				return null;
			
			TypeParser Parser = T as TypeParser;
			
			return Parser;
		}

		private bool CreateAppDomain(string lcAppDomain) 
		{
			if (lcAppDomain == null)
				lcAppDomain = "wwReflection" + Guid.NewGuid().ToString().GetHashCode().ToString("x");

			AppDomainSetup loSetup = new AppDomainSetup();

			// *** Point at current directory
			loSetup.ApplicationBase = AppDomain.CurrentDomain.BaseDirectory;

			this.LocalAppDomain = AppDomain.CreateDomain(lcAppDomain,null,loSetup);
			
			return true;
		}

		public void UnloadTypeParser() 
		{
			if (this.LocalAppDomain != null) 
			{
				AppDomain.Unload( this.LocalAppDomain );
				this.LocalAppDomain = null;
			}
		}
		#region IDisposable Members

		public void Dispose()
		{
			this.UnloadTypeParser();
		}

		#endregion
	}

}
