using System;
using Westwind.wwReflection.Tools;
using System.Runtime.InteropServices;

namespace Westwind.wwHelp
{
	
	/// <summary>
	/// This class acts as a base class wrapper around a COM object.
	/// It stores a reference to the inbound COM reference in ComInstance.
	/// Subclasses must set the ComInstance member preferably with the
	/// only constructor.
	/// </summary>
	public abstract class wwHelpComWrapper : IDisposable 
	{
		/// <summary>
		/// The underlying COM instance for this Wrapper Object
		/// </summary>
		/// <param name="ComInstance"></param>
		public object ComInstance
		{
			set 
			{
				this.m_Inst = null; // clear ref count
				this.m_Inst = value;
			}
			get 
			{ 
				return this.m_Inst; 
			}
		}
		protected object m_Inst= null;
		
		/// <summary>
		/// Releases the underlying COM instance
		/// </summary>
		public void Release()
		{
			Marshal.ReleaseComObject(m_Inst);
			m_Inst = null;		   
		}

		#region IDisposable Members

		/// <summary>
		/// Releases the COM pointer. Usually not necessary
		/// but can be used in Automation scenarios against a
		/// Top Level Com reference.
		/// </summary>
		public void Dispose() 
		{
			if (this.m_Inst != null)
				this.Release();
		}
		#endregion
	}


	/// <summary>
	/// Topic Types used for loading topics
	/// </summary>
	public enum TopicTypes 
	{
		Pk = 0,
		TopicTitle = 2,
		NamespaceSignature = 4
	}

	/// <summary>
	/// Status display mechanims used on the form to display
	/// messages with teh StatusBarMessage() method.
	/// </summary>
	public enum StatusTypes 
	{
		StatusBar = 0,
		ProgressBar = 2,
		StatusWindow = 3,
		WaitWindow = 1
	}

	/// <summary>
	/// The Help Builder Form's View modes for editing or preview
	/// </summary>
	public enum ViewerModes 
	{
		EditForm = 0,
		HtmlPreview = 1
	}
}
