using System;
using System.IO;
using System.Text;
using System.Globalization;

using System.Reflection;

namespace Westwind.wwReflection.Tools
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class wwUtils
	{

		

		/// <summary>
		/// Searches one string into another string and replaces all occurences with
		/// a blank character.
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.StrTran("Joe Doe", "o");		//returns "J e D e" :)
		/// </pre>
		/// </summary>
		/// <param name="cSearchIn"> </param>
		/// <param name="cSearchFor"> </param>
		public static string StrTran(string SearchIn, string SearchFor)
		{
			//Create the StringBuilder
			StringBuilder sb = new StringBuilder(SearchIn);
			
			//Call the Replace() method of the StringBuilder
			return sb.Replace(SearchFor," ").ToString();
		}

		/// <summary>
		/// Searches one string into another string and replaces all occurences with
		/// a third string.
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.StrTran("Joe Doe", "o", "ak");		//returns "Jake Dake" 
		/// </pre>
		/// </summary>
		/// <param name="cSearchIn"> </param>
		/// <param name="cSearchFor"> </param>
		/// <param name="cReplaceWith"> </param>
		public static string StrTran(string cSearchIn, string cSearchFor, string cReplaceWith)
		{
			//Create the StringBuilder
			StringBuilder sb = new StringBuilder(cSearchIn);

			//There is a bug in the replace method of the StringBuilder
			sb.Replace(cSearchFor, cReplaceWith);

			//Call the Replace() method of the StringBuilder and specify the string to replace with
			return sb.Replace(cSearchFor, cReplaceWith).ToString();
		}

		/// Searches one string into another string and replaces each occurences with
		/// a third string. The fourth parameter specifies the starting occurence and the 
		/// number of times it should be replaced
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.StrTran("Joe Doe", "o", "ak", 2, 1);		//returns "Joe Dake" 
		/// </pre>
		public static string StrTran(string cSearchIn, string cSearchFor, string cReplaceWith, bool ReplaceFirstOnly)
		{
			
			int At = cSearchIn.IndexOf(cSearchFor);
			if (At < 0 )
				return "";

			//Create the StringBuilder
			StringBuilder sb = new StringBuilder(cSearchIn);

			//There is a bug in the replace method of the StringBuilder
			return sb.Replace(cSearchFor, cReplaceWith,At,cSearchFor.Length).ToString(); 
		}

 		
		/// <summary>
		/// Returns Proper case for a string
		/// </summary>
		/// <param name="Input"></param>
		/// <returns></returns>
		public static string ProperCase(string Input) 
		{
			return System.Threading.Thread.CurrentThread.
				CurrentCulture.TextInfo.ToTitleCase(Input.ToLower());
		}

		/// <summary>
		/// Receives a string along with starting and ending delimiters and returns the 
		/// part of the string between the delimiters. Receives a beginning occurence
		/// to begin the extraction from and also receives a flag (0/1) where 1 indicates
		/// that the search should be case insensitive.
		/// <pre>
		/// Example:
		/// string cExpression = "JoeDoeJoeDoe";
		/// VFPToolkit.strings.StrExtract(cExpression, "o", "eJ", 1, 0);		//returns "eDo"
		/// </pre>
		/// </summary>
		public static string StrExtract(string cSearchExpression, string cBeginDelim, string cEndDelim, int nBeginOccurence, int nFlags)
		{

			string cstring = cSearchExpression;
			string cb = cBeginDelim;
			string ce = cEndDelim;
			//string lcRetVal = "";

			if (nFlags == 1) 
			{
				cb = cb.ToLower();
				ce = ce.ToLower();
				cstring = cstring.ToLower();
			}

			int lnAt = cSearchExpression.IndexOf(cb,0);
			if (lnAt < 0)
				return "";

			int lnAtCut = lnAt + cb.Length ;

			int lnAt2  = cSearchExpression.IndexOf(ce,lnAtCut);
			if (lnAt2 < 0)
				return "";

			return cSearchExpression.Substring(lnAtCut,lnAt2 - lnAtCut);
		}

		/// <summary>
		/// Receives a string and a delimiter as parameters and returns a string starting 
		/// from the position after the delimiter
		/// <pre>
		/// Example:
		/// string cExpression = "JoeDoeJoeDoe";
		/// VFPToolkit.strings.StrExtract(cExpression, "o");		//returns "eDoeJoeDoe"
		/// </pre>
		/// </summary>
		/// <param name="cSearchExpression"> </param>
		/// <param name="cBeginDelim"> </param>
		public static string StrExtract(string cSearchExpression, string cBeginDelim)
		{
			int nbpos = At(cBeginDelim, cSearchExpression);
			return cSearchExpression.Substring(nbpos + cBeginDelim.Length - 1);
		}

		/// <summary>
		/// Receives a string along with starting and ending delimiters and returns the 
		/// part of the string between the delimiters
		/// <pre>
		/// Example:
		/// string cExpression = "JoeDoeJoeDoe";
		/// VFPToolkit.strings.StrExtract(cExpression, "o", "eJ");		//returns "eDo"
		/// </pre>
		/// </summary>
		/// <param name="cSearchExpression"> </param>
		/// <param name="cBeginDelim"> </param>
		/// <param name="cEndDelim"> </param>
		public static string StrExtract(string cSearchExpression, string cBeginDelim, string cEndDelim)
		{
			return StrExtract(cSearchExpression, cBeginDelim, cEndDelim, 1, 0);
		}

		/// <summary>
		/// Receives a string along with starting and ending delimiters and returns the 
		/// part of the string between the delimiters. It also receives a beginning occurence
		/// to begin the extraction from.
		/// <pre>
		/// Example:
		/// string cExpression = "JoeDoeJoeDoe";
		/// VFPToolkit.strings.StrExtract(cExpression, "o", "eJ", 2);		//returns ""
		/// </pre>
		/// </summary>
		/// <param name="cSearchExpression"> </param>
		/// <param name="cBeginDelim"> </param>
		/// <param name="cEndDelim"> </param>
		/// <param name="nBeginOccurence"> </param>
		public static string StrExtract(string cSearchExpression, string cBeginDelim, string cEndDelim, int nBeginOccurence)
		{
			return StrExtract(cSearchExpression, cBeginDelim, cEndDelim, nBeginOccurence, 0);
		}


		/// Private Implementation: This is the actual implementation of the At() and RAt() functions. 
		/// Receives two strings, the expression in which search is performed and the expression to search for. 
		/// Also receives an occurence position and the mode (1 or 0) that specifies whether it is a search
		/// from Left to Right (for At() function)  or from Right to Left (for RAt() function)
		private static int __at(string cSearchFor, string cSearchIn, int nOccurence, int nMode)
		{
			//In this case we actually have to locate the occurence
			int i = 0;
			int nOccured = 0;
			int nPos = 0;
			if (nMode == 1) {nPos = 0;}
			else {nPos = cSearchIn.Length;}

			//Loop through the string and get the position of the requiref occurence
			for (i=1;i<=nOccurence;i++)
			{
				if(nMode == 1) {nPos = cSearchIn.IndexOf(cSearchFor,nPos);}
				else {nPos = cSearchIn.LastIndexOf(cSearchFor,nPos);}

				if (nPos < 0)
				{
					//This means that we did not find the item
					break;
				}
				else
				{
					//Increment the occured counter based on the current mode we are in
					nOccured++;

					//Check if this is the occurence we are looking for
					if (nOccured == nOccurence)
					{
						return nPos + 1;
					}
					else
					{
						if(nMode == 1) {nPos++;}
						else {nPos--;}

					}
				}
			}
			//We never found our guy if we reached here
			return 0;
		}
		/// <summary>
		/// Receives two strings as parameters and searches for one string within another. 
		/// If found, returns the beginning numeric position otherwise returns 0
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.At("D", "Joe Doe");	//returns 5
		/// </pre>
		/// </summary>
		/// <param name="cSearchFor"> </param>
		/// <param name="cSearchIn"> </param>
		public static int At(string cSearchFor, string cSearchIn)
		{
			return cSearchIn.IndexOf(cSearchFor) + 1;
		}

		/// <summary>
		/// Receives two strings and an occurence position (1st, 2nd etc) as parameters and 
		/// searches for one string within another for that position. 
		/// If found, returns the beginning numeric position otherwise returns 0
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.At("o", "Joe Doe", 1);	//returns 2
		/// VFPToolkit.strings.At("o", "Joe Doe", 2);	//returns 6
		/// </pre>
		/// </summary>
		/// <param name="cSearchFor"> </param>
		/// <param name="cSearchIn"> </param>
		/// <param name="nOccurence"> </param>
		public static int At(string cSearchFor, string cSearchIn, int nOccurence)
		{
			return __at(cSearchFor, cSearchIn, nOccurence, 1);
		}


		/// <summary>
		/// Receives two strings as parameters and searches for one string within another. 
		/// This function ignores the case and if found, returns the beginning numeric position 
		/// otherwise returns 0
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.AtC("d", "Joe Doe");	//returns 5
		/// </pre>
		/// </summary>
		/// <param name="cSearchFor"> </param>
		/// <param name="cSearchIn"> </param>
		public static int AtC(string cSearchFor, string cSearchIn)
		{
			return cSearchIn.ToLower().IndexOf(cSearchFor.ToLower()) + 1;
		}

		/// <summary>
		/// Receives two strings and an occurence position (1st, 2nd etc) as parameters and 
		/// searches for one string within another for that position. This function ignores the
		/// case of both the strings and if found, returns the beginning numeric position 
		/// otherwise returns 0.
		/// <pre>
		/// Example:
		/// VFPToolkit.strings.AtC("d", "Joe Doe", 1);	//returns 5
		/// VFPToolkit.strings.AtC("O", "Joe Doe", 2);	//returns 6
		/// </pre>
		/// </summary>
		/// <param name="cSearchFor"> </param>
		/// <param name="cSearchIn"> </param>
		/// <param name="nOccurence"> </param>
		public static int AtC(string cSearchFor, string cSearchIn, int nOccurence)
		{
			return __at(cSearchFor.ToLower(), cSearchIn.ToLower(), nOccurence, 1);
		}

		#region Reflection Helper Code
		/// <summary>
		/// Binding Flags constant to be reused for all Reflection access methods.
		/// </summary>
		public const BindingFlags MemberAccess = 
			BindingFlags.Public | BindingFlags.NonPublic | 
			BindingFlags.Static | BindingFlags.Instance | BindingFlags.IgnoreCase ;



		/// <summary>
		/// Retrieve a dynamic 'non-typelib' property
		/// </summary>
		/// <param name="Object">Object to make the call on</param>
		/// <param name="Property">Property to retrieve</param>
		/// <returns>Object - cast to proper type</returns>
		public static object GetProperty(object Object,string Property)
		{
			return Object.GetType().GetProperty(Property,wwUtils.MemberAccess).GetValue(Object,null);
		}

		/// <summary>
		/// Retrieve a dynamic 'non-typelib' field
		/// </summary>
		/// <param name="Object">Object to retreve Field from</param>
		/// <param name="Property">name of the field to retrieve</param>
		/// <returns></returns>
		public static object GetField(object Object,string Property)
		{
			return Object.GetType().GetField(Property,wwUtils.MemberAccess).GetValue(Object);
		}

		/// <summary>
		/// Returns a property or field value using a base object and sub members including . syntax.
		/// For example, you can access: this.oCustomer.oData.Company with (this,"oCustomer.oData.Company")
		/// </summary>
		/// <param name="Parent">Parent object to 'start' parsing from.</param>
		/// <param name="Property">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyEx(object Parent, string Property) 
		{
			MemberInfo Member = null;

			Type Type = Parent.GetType();

			int lnAt = Property.IndexOf(".");
			if ( lnAt < 0) 
			{
				if (Property == "this" || Property == "me")
					return Parent;

				// *** Get the member
				Member = Type.GetMember(Property,wwUtils.MemberAccess)[0];
				if (Member.MemberType == MemberTypes.Property )
					return ((PropertyInfo) Member).GetValue(Parent,null);
				else
					return ((FieldInfo) Member).GetValue(Parent);
			}

			// *** Walk the . syntax - split into current object (Main) and further parsed objects (Subs)
			string Main = Property.Substring(0,lnAt);
			string Subs = Property.Substring(lnAt+1);

			// *** Retrieve the current property
			Member = Type.GetMember(Main,wwUtils.MemberAccess)[0];

			object Sub;
			if (Member.MemberType == MemberTypes.Property )
			{
				// *** Get its value
				Sub = ((PropertyInfo) Member).GetValue(Parent,null);

			}
			else
			{
				Sub = ( (FieldInfo) Member).GetValue(Parent);

			}

			// *** Recurse further into the sub-properties (Subs)
			return wwUtils.GetPropertyEx(Sub,Subs);
		}

		/// <summary>
		/// Sets the property on an object.
		/// </summary>
		/// <param name="Object">Object to set property on</param>
		/// <param name="Property">Name of the property to set</param>
		/// <param name="Value">value to set it to</param>
		public static void SetProperty(object Object,string Property,object Value)
		{
			Object.GetType().GetProperty(Property,wwUtils.MemberAccess).SetValue(Object,Value,null);
		}

		/// <summary>
		/// Sets the field on an object.
		/// </summary>
		/// <param name="Object">Object to set property on</param>
		/// <param name="Property">Name of the field to set</param>
		/// <param name="Value">value to set it to</param>
		public static void SetField(object Object,string Property,object Value)
		{
			Object.GetType().GetField(Property,wwUtils.MemberAccess).SetValue(Object,Value);
		}

		/// <summary>
		/// Sets the value of a field or property via Reflection. This method alws 
		/// for using '.' syntax to specify objects multiple levels down.
		/// 
		/// wwUtils.SetPropertyEx(this,"Invoice.LineItemsCount",10)
		/// 
		/// which would be equivalent of:
		/// 
		/// this.Invoice.LineItemsCount = 10;
		/// </summary>
		/// <param name="Object Parent">
		/// Object to set the property on.
		/// </param>
		/// <param name="String Property">
		/// Property to set. Can be an object hierarchy with . syntax.
		/// </param>
		/// <param name="Object Value">
		/// Value to set the property to
		/// </param>
		public static object SetPropertyEx(object Parent, string Property,object Value) 
		{
			Type Type = Parent.GetType();
			MemberInfo Member = null;

			// *** no more .s - we got our final object
			int lnAt = Property.IndexOf(".");
			if ( lnAt < 0) 
			{
				Member = Type.GetMember(Property,wwUtils.MemberAccess)[0];
				if ( Member.MemberType == MemberTypes.Property ) 
				{

					((PropertyInfo) Member).SetValue(Parent,Value,null);
					return null;
				}
				else 
				{
					((FieldInfo) Member).SetValue(Parent,Value);
					return null;				   
				}
			}	

			// *** Walk the . syntax
			string Main = Property.Substring(0,lnAt);
			string Subs = Property.Substring(lnAt+1);
			Member = Type.GetMember(Main,wwUtils.MemberAccess)[0];

			object Sub;
			if (Member.MemberType == MemberTypes.Property)
				Sub = ((PropertyInfo) Member).GetValue(Parent,null);
			else
				Sub = ((FieldInfo) Member).GetValue(Parent);

			// *** Recurse until we get the lowest ref
			SetPropertyEx(Sub,Subs,Value);
			return null;
		}

		/// <summary>
		/// Wrapper method to call a 'dynamic' (non-typelib) method
		/// on a COM object
		/// </summary>
		/// <param name="Params"></param>
		/// 1st - Method name, 2nd - 1st parameter, 3rd - 2nd parm etc.
		/// <returns></returns>
		public static object CallMethod(object Object,string Method, params object[] Params)
		{
			return Object.GetType().InvokeMember(Method,wwUtils.MemberAccess | BindingFlags.InvokeMethod,null,Object,Params);
			//return Object.GetType().GetMethod(Method,wwUtils.MemberAccess | BindingFlags.InvokeMethod).Invoke(Object,Params);
		}

		/// <summary>
		/// Creates an instance from a type by calling the parameterless constructor.
		/// 
		/// Note this will not work with COM objects - continue to use the Activator.CreateInstance
		/// for COM objects.
		/// <seealso>Class wwUtils</seealso>
		/// </summary>
		/// <param name="TypeToCreate">
		/// The type from which to create an instance.
		/// </param>
		/// <returns>object</returns>
		public object CreateInstanceFromType( Type TypeToCreate ) 
		{
			Type[]  Parms = Type.EmptyTypes;
			return TypeToCreate.GetConstructor(Parms).Invoke(null);
		}

		/// <summary>
		/// Converts a type to string if possible. This method supports an optional culture generically on any value.
		/// It calls the ToString() method on common types and uses a type converter on all other objects
		/// if available
		/// </summary>
		/// <param name="RawValue">The Value or Object to convert to a string</param>
		/// <param name="Culture">Culture for numeric and DateTime values</param>
		/// <returns>string</returns>
		public static string TypedValueToString(object RawValue,CultureInfo Culture) 
		{
			Type ValueType = RawValue.GetType();
			string Return = null;

			if (ValueType == typeof(string) ) 
				Return = RawValue.ToString();
			else if ( ValueType == typeof(int) || ValueType == typeof(decimal) || 
				ValueType == typeof(double) || ValueType == typeof(float))
				Return = string.Format(Culture.NumberFormat,"{0}",RawValue);
			else if(ValueType == typeof(DateTime))
				Return =  string.Format(Culture.DateTimeFormat,"{0}",RawValue);
			else if(ValueType == typeof(bool))
				Return = RawValue.ToString();
			else if(ValueType == typeof(byte))
				Return = RawValue.ToString();
			else if(ValueType.IsEnum)
				Return = RawValue.ToString();
			else 
			{
				// Any type that supports a type converter
				System.ComponentModel.TypeConverter converter = 
					System.ComponentModel.TypeDescriptor.GetConverter( ValueType );
				if (converter != null && converter.CanConvertTo(typeof(string)) )
					Return = converter.ConvertToString(null,Culture,RawValue );
				else
					// Last resort - just call ToString() on unknown type
					Return = RawValue.ToString();

			}
		
			return Return;
		}

		/// <summary>
		/// Converts a type to string if possible. This method uses the current culture for numeric and DateTime values.
		/// It calls the ToString() method on common types and uses a type converter on all other objects
		/// if available.
		/// </summary>
		/// <param name="RawValue">The Value or Object to convert to a string</param>
		/// <param name="Culture">Culture for numeric and DateTime values</param>
		/// <returns>string</returns>
		public static string TypedValueToString(object RawValue) 
		{
			return TypedValueToString(RawValue,CultureInfo.CurrentCulture);
		}

		/// <summary>
		/// Turns a string into a typed value. Useful for auto-conversion routines
		/// like form variable or XML parsers.
		/// <seealso>Class wwUtils</seealso>
		/// </summary>
		/// <param name="SourceString">
		/// The string to convert from
		/// </param>
		/// <param name="TargetType">
		/// The type to convert to
		/// </param>
		/// <param name="Culture">
		/// Culture used for numeric and datetime values.
		/// </param>
		/// <returns>object. Throws exception if it cannot be converted.</returns>
		public static object StringToTypedValue(string SourceString, Type TargetType, CultureInfo Culture ) 
		{
			object Result = null;

			if ( TargetType == typeof(string) )
				Result = SourceString;
			else if (TargetType == typeof(int))  
				Result = int.Parse( SourceString, System.Globalization.NumberStyles.Integer, Culture.NumberFormat );			
			else if (TargetType  == typeof(byte) )  
				Result = Convert.ToByte(SourceString);				
			else if (TargetType  == typeof(decimal))  
				Result = Decimal.Parse(SourceString,System.Globalization.NumberStyles.Any, Culture.NumberFormat);
			else if (TargetType  == typeof(double))    
				Result = Double.Parse( SourceString,System.Globalization.NumberStyles.Any, Culture.NumberFormat);				
			else if (TargetType == typeof(bool)) 
			{
				if (SourceString.ToLower() == "true" || SourceString.ToLower() == "on" || SourceString == "1")
					Result = true;
				else
					Result = false;
			}
			else if (TargetType == typeof(DateTime))  
				Result = Convert.ToDateTime(SourceString,Culture.DateTimeFormat);	
			else if (TargetType.IsEnum)
				Result = Enum.Parse(TargetType,SourceString);
			else   
			{
				System.ComponentModel.TypeConverter converter = System.ComponentModel.TypeDescriptor.GetConverter(TargetType);
				if (converter != null && converter.CanConvertFrom(typeof(string)) )
					Result = converter.ConvertFromString( null, Culture, SourceString );
				else  
				{
					System.Diagnostics.Debug.Assert(false,"Type Conversion not handled in StringToTypedValue\r\n" + 
						TargetType.Name + " " + SourceString );
					throw(new ApplicationException("Type Conversion not handled in StringToTypedValue"));
				}
			}

			return Result;
		}

		/// <summary>
		/// Turns a string into a typed value. Useful for auto-conversion routines
		/// like form variable or XML parsers.
		/// </summary>
		/// <param name="SourceString">The input string to convert</param>
		/// <param name="TargetType">The Type to convert it to</param>
		/// <returns>object reference. Throws Exception if type can not be converted</returns>
		public static object StringToTypedValue(string SourceString, Type TargetType ) 
		{
			return StringToTypedValue(SourceString,TargetType,CultureInfo.CurrentCulture);
		}

		#endregion

		#region COM Reflection Helper Code
		
		/// <summary>
		/// Retrieve a dynamic 'non-typelib' property
		/// </summary>
		/// <param name="Object">Object to make the call on</param>
		/// <param name="Property">Property to retrieve</param>
		/// <returns></returns>
		public static object GetPropertyCom(object Object,string Property)
		{
				return Object.GetType().InvokeMember(Property,wwUtils.MemberAccess | BindingFlags.GetProperty,null,
				Object,null);
		}

		
		/// <summary>
		/// Returns a property or field value using a base object and sub members including . syntax.
		/// For example, you can access: this.oCustomer.oData.Company with (this,"oCustomer.oData.Company")
		/// </summary>
		/// <param name="Parent">Parent object to 'start' parsing from.</param>
		/// <param name="Property">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyExCom(object Parent, string Property) 
		{

			Type Type = Parent.GetType();

			int lnAt = Property.IndexOf(".");
			if ( lnAt < 0) 
			{
				if (Property == "this" || Property == "me")
					return Parent;

				// *** Get the member
				return Parent.GetType().InvokeMember(Property,wwUtils.MemberAccess | BindingFlags.GetProperty | BindingFlags.GetField,null,
					Parent,null);
			}

			// *** Walk the . syntax - split into current object (Main) and further parsed objects (Subs)
			string Main = Property.Substring(0,lnAt);
			string Subs = Property.Substring(lnAt+1);

			object Sub = Parent.GetType().InvokeMember(Main,wwUtils.MemberAccess | BindingFlags.GetProperty | BindingFlags.GetField,null,
				Parent,null);

			// *** Recurse further into the sub-properties (Subs)
			return wwUtils.GetPropertyExCom(Sub,Subs);
		}

		/// <summary>
		/// Sets the property on an object.
		/// </summary>
		/// <param name="Object">Object to set property on</param>
		/// <param name="Property">Name of the property to set</param>
		/// <param name="Value">value to set it to</param>
		public static void SetPropertyCom(object Object,string Property,object Value)
		{
			Object.GetType().InvokeMember(Property,wwUtils.MemberAccess | BindingFlags.SetProperty,null,Object,new object[1] { Value } );
			//GetProperty(Property,wwUtils.MemberAccess).SetValue(Object,Value,null);
		}

		/// <summary>
		/// Sets the value of a field or property via Reflection. This method alws 
		/// for using '.' syntax to specify objects multiple levels down.
		/// 
		/// wwUtils.SetPropertyEx(this,"Invoice.LineItemsCount",10)
		/// 
		/// which would be equivalent of:
		/// 
		/// this.Invoice.LineItemsCount = 10;
		/// </summary>
		/// <param name="Object Parent">
		/// Object to set the property on.
		/// </param>
		/// <param name="String Property">
		/// Property to set. Can be an object hierarchy with . syntax.
		/// </param>
		/// <param name="Object Value">
		/// Value to set the property to
		/// </param>
		public static object SetPropertyExCom(object Parent, string Property,object Value) 
		{
			Type Type = Parent.GetType();

			int lnAt = Property.IndexOf(".");
			if ( lnAt < 0) 
			{
				// *** Set the member
				Parent.GetType().InvokeMember(Property,wwUtils.MemberAccess | BindingFlags.SetProperty | BindingFlags.SetField,null,
					Parent,new object[1] { Value } );

				return null;
			}

			// *** Walk the . syntax - split into current object (Main) and further parsed objects (Subs)
			string Main = Property.Substring(0,lnAt);
			string Subs = Property.Substring(lnAt+1);


			object Sub = Parent.GetType().InvokeMember(Main,wwUtils.MemberAccess | BindingFlags.GetProperty | BindingFlags.GetField,null,
				Parent,null);

			return SetPropertyExCom(Sub,Subs,Value);
		}


		/// <summary>
		/// Wrapper method to call a 'dynamic' (non-typelib) method
		/// on a COM object
		/// </summary>
		/// <param name="Params"></param>
		/// 1st - Method name, 2nd - 1st parameter, 3rd - 2nd parm etc.
		/// <returns></returns>
		public static object CallMethodCom(object Object,string Method, params object[] Params)
		{
			return Object.GetType().InvokeMember(Method,wwUtils.MemberAccess | BindingFlags.InvokeMethod,null,Object,Params);
		}

		#endregion


	}
}
